#! /usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

my %lex = ( 'Main' => 'Main',
	    'Error' => 'Error',
	    'Continue' => 'Continue',
	    'Please Log In' => 'Please Log In',
	    

    );



my $self = 'attendcheck_local.pl';


use DBI;
use CGI;
use CGI::Session;
use Number::Format qw(:all);
use Cwd;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}
eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;


# Absent/Late strings for libattend library.
my %lexi = ('Absent' => 'Absent',
	    'Late' => 'Late'
    );


# load attendance library
eval require "../lib/libattend.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currdate = "$year-$month-$day";

# Now get the students in this school
my %studs; # studs{dbase}{studnum} = record;
my $sth = $dbh->prepare("select * from special where school = ?");
$sth->execute( $schooldbase );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my $ref = $sth->fetchrow_hashref ) {
    my %r = %$ref;

    my $studnum = $r{studnum};
    if ( $r{studnum_local} ) {
	$studnum = $r{studnum_local};
    }
    
    $studs{ $r{school} }{$studnum} = $ref;

}


# Loop to check.
=head
foreach my $db ( sort keys %studs ) {
    foreach my $studnum ( keys %{ $studs{$db} } ) {
	my %r = %{ $studs{$db}{$studnum} };
	if ( $r{studnum_local} ) {
	    print "LOCAL:$r{studnum_local} - $r{studnum}\n";
	    sleep 2;
	}
	print qq{DB:$db SN:$studnum REC:$r{lastname} $r{firstname}<br>\n};
    }
}
=cut



# Page Header
my $title = "IEP Attendance Report";

print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};

print qq{$chartype\n</head><body style="padding:1em;"><a name="top"></a>\n};
print qq{[ <a href="$homepage">Main</a> \n};
print qq{]\n};

print qq{<h1>$title</h1>\n};


# Open the local school database
my $dsn1 = "DBI:$dbtype:dbname=$schooldbase";
my $dbh1 = DBI->connect($dsn1,$guser,$gpassword);
$dbh->{mysql_enable_utf8} = 1;

my $first = 1;

my $db = $schooldbase; # since values below use the $db value.


# Load Configuration Variables;
my $sth = $dbh1->prepare("select id, datavalue from conf_system where filename = 'admin'");
$sth->execute;
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
while (	my ($id, $datavalue) = $sth->fetchrow ) {
    eval $datavalue;
    if ( $@ ) {
	print "$lex{Error}: $@<br>\n";
	die "$lex{Error}: $@\n";
    }
}


my $startdate = $schoolstart;
my $enddate = $currdate;

my $termstart = $g_MTrackTerm{1}{1}{start};
# print qq{Term Start: $termstart<br>\n};
$startdate = $termstart;

    
my %schooldays = mkSchoolDays($startdate, $enddate, $dbh1);
# These are the school days open in each month, holidays excluded from the count.
    
# Loop through students in this school and get their attendance.
my %sort;
foreach my $studnum ( keys %{ $studs{$db} } ) {
    my %r = %{ $studs{$db}{$studnum} };
    $sort{"$r{lastname}$r{firstname}$studnum"} = $studnum;
}

# Student Loop for this school
foreach my $key ( sort keys %sort ) {
    my $studnum = $sort{$key};

    my %r = %{ $studs{$db}{$studnum} };

    if ( $r{studnum_local} ) {
	$studnum = $r{studnum_local};
    }

    my $ppd = $g_ppd{ $r{grade} };
    if ( not $ppd ) { 
	print qq{<div>Missing Attendance Periods Per Day - SN:$studnum $r{firstname} $r{lastname}};
	print qq{GR:$r{grade} DB:$db - SETTING PPD to 2<br>\n}; 
	$ppd = 2;
    }


    if ( $first ) { # print heading.
	print qq{<h3>$dbase{$db} ($db)</h3>\n};
	print qq{<table cellpadding="3" border="1" cellspacing="0">\n};
	print qq{<tr><th>Name</th>};
	foreach my $yrmo ( sort keys %schooldays ) {
	    print qq{<th>$yrmo</th>};
	}
	print qq{</tr>\n};
	$first = 0;
    }

    print qq{<tr><td><b>$r{lastname}</b>, $r{firstname}</td>\n};
	    

    # Calculate the days enrolled for this student
    my $ref = calcMonthlyEnrollment( $studnum, $startdate, $enddate, $dbh1);
    my %enrol = %$ref;  # enrol{year-month} = val
    if ( not $ref ) { #student not enrolled.
	my $skip = scalar keys %schooldays;
	print qq{<td colspan="$skip" style="background-color:salmon;font-weight:bold;">};
	print qq{Enrollment Error: No Enrollment this school year</td></tr>\n};
	next; 
    } 

    my ($currmonth,$prevmonth);

    # Loop through months
    foreach my $yrmo ( sort keys %schooldays ) {

	my $abslate = calcMonthlyAttendance( $studnum, $yrmo, $ppd, '', \%lexi, $dbh1);
	my ($absent,$late ) = split(':', $abslate);

	    
	my $pctAttendance;
	if ( $enrol{$yrmo}{days} ) {
	    my $present = $enrol{$yrmo}{days} - $absent;
	    $pctAttendance = round($present * 100 / $enrol{$yrmo}{days}, 1);
	} else { # no enrollment....
	    print qq{<td>No Enrol</td>};
	    next; # month
	}

	$prevmonth = $currmonth;
	$currmonth = $pctAttendance;

	    
	my $color;
	if ( $pctAttendance > 70 ) { 
	    $color = 'lightgreen';
	} elsif ( $pctAttendance > 49.9 ) {
	    $color = 'yellow';
	} else {
	    $color = 'red';
	}

	print qq{<td style="background-color:$color;">$pctAttendance</td>\n};

    } # end of year-month loop
	    
    print qq{</tr>\n};


} # end of student loop

if ( $first ) {
    print qq{<h4>No Students Found</h4>\n};
} else {
    print qq{</table><p></p>\n};
}


print qq{</body></html>\n};

exit;

