#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Eoy' => 'Eoy',
	   'Configuration' => 'Configuration',
	   'Error' => 'Error',
	   'Edit' => 'Edit',
	   'View' => 'View',
	   'Name' => 'Name',
	   'Value' => 'Value',
	   'Description' => 'Description',
	   'File' => 'File',
	   'Record' => 'Record',
	   'Update' => 'Update',
	   'Updated' => 'Updated',
	   'Key' => 'Key',
	   'New' => 'New',
	   'Yes' => 'Yes',
	   'No' => 'No',
	   'Main' => 'Main',
	   'Start' => 'Start',
	   'Section' => 'Section',
	   'Files' => 'Files',
	   'Not Found' => 'Not Found',
	   'Path' => 'Path',
	   'Area' => 'Area',

	   );



my $self = 'iepConfEdit.pl';

my @types = qw( scalar scalaronoff hash hashofarray array); # data types
my $maxarrayidx = 20; # 8 array entries 0..7
my $scalar_min_length = 30; # smallest size for scalar input.
my $scalar_text_rows = 10;
my $scalar_text_cols = 65;


use DBI;
use CGI;

eval require "../ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# load config values.
my $sth = $dbh->prepare("select datavalue from conf_system where dataname = ?");
$sth->execute( 'g_EtcPath' );
my $datavalue = $sth->fetchrow;
eval $datavalue;
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


# Page Header
print qq{$doctype\n<html><head><title>$lex{Edit} $lex{Configuration}</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{$chartype\n</head><body style="margin:1em;">\n};

print qq{[ <a href="$homepage">$lex{Main}</a> | <a href="$eoypage">$lex{Eoy}</a> |\n};
print qq{ <a href="$discpage">$lex{Discipline}</a> | <a href="$self">$lex{Start}</a> ]\n};

print qq{<h1>$lex{Edit} $lex{Configuration}</h1>\n};


if ( not $arr{page} ) {
    delete $arr{page};
    showValues();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    editValues();

} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    writeValues();

} elsif ( $arr{page} == 4 ) {
    delete $arr{page};
    updateFiles();
}



#-------------
sub showValues {
#-------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }
    # Normally, only passed filename.

    $sth = $dbh->prepare("select id, dataname, datavalue, description, sectionname 
       from conf_system order by sectionname, sequenceval, dataname");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    
    print qq{<h3 style="font-size:120%;font-weight:bold;margin:0.5em 1em;">};
    print qq{$lex{Configuration} $lex{Area}: IEP</h3>\n};

    # Print Table Heading.
    print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin:0 1em;">\n};

    my ($prevsection, $currsection);
    my $first = 1;

    while ( my ( $id, $dataname,$datavalue, $description, $sectionname ) = $sth->fetchrow ) {

	# Skip date values, since set by confyear.pl script.	
	if ( $arr{filename} eq 'admin' and $sectionname eq 'schoolyear' ) { next; } #skip

	$prevsection = $currsection;
	$currsection = $sectionname;

	if ( $currsection ne $prevsection ) {
	    print qq{<tr><th></th><th>$lex{Name}</th><th>$lex{Section}</th>};
	    print qq{<th>$lex{Description}</th><th>$lex{Value}</th></tr>\n};
	}


	$dataname =~ s/^r\_//; # strip r_
	my ( $dud, $val ) = split("= ", $datavalue);
	$val =~ s/\</&lt;/g;
	$val =~ s/\>/&gt;/g;

	print qq{<tr><td><form action="$self"  method="post">\n};
	print qq{<input type="hidden" name="id" value="$id">\n};
	print qq{<input type="hidden" name="page" value="2">\n};
	print qq{<input type="submit" value="$lex{Edit}"></form></td>\n};

	print qq{<td>$dataname</td><td>$sectionname</td><td>$description</td><td>$val</td></tr>\n};
	
    }

    print qq{</table></body></html>\n};

    exit;

}  # end of showValues



#-------------
sub editValues { # edit record whose is id passed
#-------------

    #foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }
    # just id is passed.
    
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="3">\n};
    print qq{<input type="hidden" name="id" value="$arr{id}">\n};

    # Get the Record
    my $sth = $dbh->prepare("select * from conf_system where id = ?");
    $sth->execute( $arr{id} );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my %rec = %$ref;

    
    # Convert any double quotes to an entity.
    $rec{datavalue} =~ s/"/&#34;/g;

    my $type = $ref->{datatype};
    my $hashvar = $ref->{hashvar};
    my $sectionname = $ref->{sectionname};

    eval $rec{datavalue};
    if ( $@ ) {
	print $lex{Error}. " $@<br>\n";
	die $lex{Error}. " $@\n";
    }


    print qq{<table cellpadding="3" cellspacing="0" border="0" width="600">\n};


    print qq{<tr><td></td><td style="text-align:left;">\n};
    print qq{<input type="submit" value="$lex{Update} $lex{Record}"></td></tr>\n};

    # Description
    print qq{<tr><td class="bra">$lex{Description}</td>\n};
    print qq{<td><textarea name="description" rows="3" cols="100">$rec{description}</textarea></td></tr>\n};

    # Data Name
    my $dataname = $rec{dataname};
    $dataname =~ s/^r\_//; # strip leading r_
    print qq{<tr><td class="bra">$lex{Name}</td>\n<td>};
    print qq{$dataname</td></tr>\n};


    #Data Value 
    print qq{<tr><td class="bra">$lex{Value}</td>\n<td>};

    # SQL setup for getting hash values.
    $sth = $dbh->prepare("select * from conf_system where dataname = ?");

    # Now the various data types.
    if ( $type eq 'scalar' ) {

	my $varname = $rec{dataname};
	my $varvalue = $$varname; # watch $$.

	if ( $sectionname eq 'Notes' ) { # use a text area for input, edit.
	    print qq{<textarea rows="$scalar_text_rows" cols="$scalar_text_cols" name="$varname">};
	    print qq{$varvalue</textarea></td></tr>\n};

	} else {
	    my $size = length( $varvalue ) + 5;
	    if ( $size < $scalar_min_length ) { $size = $scalar_min_length; }
	    print qq{<input type="text" size="$size" name="$varname" };
	    print qq{value="$varvalue"></td></tr>\n};
	}

    } elsif ( $type eq 'scalaronoff' ) {

	my $varname = $rec{dataname};
	my $varvalue = $$varname; # watch $$.

	print qq{<select name="$varname"><option>$varvalue</option>\n};
	foreach my $val ( 0 .. 1 ) {
	    if ( $val == $varvalue ) { next; }
	    print qq{<option>$val</option>\n};
	}
	print qq{</select> 1=$lex{Yes}, 0=$lex{No}</td></tr>\n};


    } elsif ( $type eq 'array' ) {

	my $varname = $rec{dataname};
        print qq{<table cellpadding="3" cellspacing="0" border="0">\n};

	my $maxsize; # biggest value for characters in values
	my $counter = 1;

	foreach my $val ( @{ $varname } ) {

	    print qq{<tr><td>};
	    
	    my $size = length( $val ) + 5;
	    if ( $size > $maxsize ) { $maxsize = $size; }
	    if ( $size < 40 ) { # use input method
		print qq{<input type="text" size="$size" name="$counter" value="$val">\n};
	    } else { # use textarea method
		print qq{<textarea name="$counter" rows="6" cols="80">$val</textarea>\n};
	    }
	    print qq{</td></tr>\n};
	    $counter++;
	}
	print qq{</table>\n};
	print qq{</td></tr>\n};

	# blank row
	print qq{<tr><td colspan="2">&nbsp;</td></tr>\n};

	# New Value	
	print qq{<tr><td class="ra">$lex{New} $lex{Value}</td>\n<td>};
	if ( $maxsize > 30 ) { # use textarea 
	    print qq{<textarea name="newvalue" rows="6" cols="80">$varname->{$key}</textarea>\n};
	} else { # normal input field
	    print qq{<input type="text" name="newvalue" size="$maxsize">\n};
	}
	print qq{</td></tr>\n};


    } elsif ( $type eq 'hash' ) {

	# Check for a source hash for keys
	my $sourcehash = $rec{hashvar};
	if ( $sourcehash ) { # load that variable to get the keys.
	    # Get the related hashofarray Record
	    $sth->execute( $sourcehash );
	    my $ref = $sth->fetchrow_hashref;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    eval $ref->{datavalue}; # evaluate $sourcehash.
	}

	my @keys;
	foreach my $key ( sort {$a <=> $b} keys %{ $sourcehash } ) {
	    push @keys, $key;
	}


	my $varname = $rec{dataname};
        print qq{<table cellpadding="3" cellspacing="0" border="0">\n};

	my $maxsize; # biggest value for characters in values
	foreach my $key ( sort keys %{ $varname } ) {

	    # print the key
	    print qq{<tr><td>$key</td><td>\n};
	    

	    # decide on entry method
	    if ( @keys ) { # use the select method
		print qq{<select name="$key"><option>$varname->{$key}</option>\n};
		foreach my $skey ( @keys ) { 
		    if ( $skey ne $varname->{$key} ) { # don't show duplicates
			print qq{<option>$skey</option>};
		    }
		}
		print qq{<option></option></select>\n};

	    } else {
		my $size = length( $varname->{$key} ) + 5;
		if ( $size > $maxsize ) { $maxsize = $size; } 
		if ( $size < 30 ) { # use input method
		    print qq{<input type="text" size="$size" name="$key" value="$varname->{$key}">\n};
		} else { # use textarea method
		    print qq{<textarea name="$key" rows="6" cols="80">$varname->{$key}</textarea>\n};
		}
	    }

	    print qq{</td></tr>\n};
	}
	print qq{</table>\n};

	print qq{</td></tr>\n};

	# New Key/Value
	print qq{<tr><td colspan="2">&nbsp;</td></tr>\n}; # blank row
	
	print qq{<tr><td class="ra">$lex{New} $lex{Key}</td>\n};
	print qq{<td><input type="text" name="newkey" size="5"></td></tr>\n};

	print qq{<tr><td class="ra">$lex{New} $lex{Value}</td>\n<td>};
	if ( $maxsize > 40 ) { # use textarea 
	    print qq{<textarea name="newvalue" rows="6" cols="80">$varname->{$key}</textarea>\n};
	} else { # normal input field
	    print qq{<input type="text" name="newvalue" size="$maxsize">\n};
	}
	print qq{</td></tr>\n};


    } elsif ( $type eq 'hashofarray' ) {

	# Get the related hash values
	$sth->execute( $hashvar );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $sourceref = $sth->fetchrow_hashref;
	eval $sourceref->{datavalue};
	
	# Loop Through all Hash values.
	my $varname = $rec{dataname};
        print qq{<table cellpadding="3" cellspacing="0" border="0">\n};
	foreach my $key ( sort { $a <=> $b } keys %{ $varname } ) {

	    my $arrayref = $varname->{$key};

	    print qq{<tr><td>$key</td>\n<td>};
	    my $arrayidx = 0;
	    foreach my $val ( @$arrayref ) {
		my $size = length( $val ) + 5;
		print qq{$arrayidx.<input type="text" size="$size" name="$key:$arrayidx" value="$val"><br>\n};
		$arrayidx++;
	    }
	    print qq{</td></tr>\n};
	}

	print qq{</table>\n};
	print qq{</td></tr>\n};


	# New Key/Value
	print qq{<tr><td colspan="2">&nbsp;</td></tr>\n}; # blank row

	print qq{<tr><td class="ra">$lex{New} $lex{Key}</td>\n};
	print qq{<td><input type="text" name="newkey" size="5"></td></tr>\n};

	print qq{<tr><td class="ra">$lex{New} $lex{Value}</td><td>\n};
	for my $i ( 0 .. $maxarrayidx ) {
	    print qq{$i.<input type="text" name="NEW:$i" size="20"><br>\n};
	}
	print qq{</td></tr>\n};


    } elsif ( $type eq 'arrayofhash' ) {

	# Get the related hash values
	$sth->execute( $hashvar );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $sourceref = $sth->fetchrow_hashref;
	eval $sourceref->{datavalue};
	
	# Loop Through all Hash values.
	my $varname = $rec{dataname};
        print qq{<table cellpadding="3" cellspacing="0" border="0">\n};
	foreach my $key ( sort { $a <=> $b } keys %{ $varname } ) {

	    my $arrayref = $varname->{$key};

	    print qq{<tr><td>$key</td>\n<td>};
	    my $arrayidx = 0;
	    foreach my $val ( @$arrayref ) {
		my $size = length( $val ) + 5;
		print qq{$arrayidx.<input type="text" size="$size" name="$key:$arrayidx" value="$val"><br>\n};
		$arrayidx++;
	    }
	    print qq{</td></tr>\n};
	}

	print qq{</table>\n};
	print qq{</td></tr>\n};


	# New Key/Value
	print qq{<tr><td colspan="2">&nbsp;</td></tr>\n}; # blank row

	print qq{<tr><td class="ra">$lex{New} $lex{Key}</td>\n};
	print qq{<td><input type="text" name="newkey" size="5"></td></tr>\n};

	print qq{<tr><td class="ra">$lex{New} $lex{Value}</td><td>\n};
	for my $i ( 0 .. $maxarrayidx ) {
	    print qq{$i.<input type="text" name="NEW:$i" size="20"><br>\n};
	}
	print qq{</td></tr>\n};


    } # end of arrayofhash type


    print qq{<tr><td></td><td style="text-align:left;">\n};
    print qq{<input type="submit" value="$lex{Update} $lex{Record}"></td></tr>\n};
    print qq{</table><p></p>\n};
    print qq{</form></body></html>\n};

    exit;


} # end of editValues



#--------------
sub writeValues {
#--------------
    
    use Data::Dumper;
    $Data::Dumper::Purity = 1;
    $Data::Dumper::Indent = 0;

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    # Get the Id value
    my $id = $arr{id};
    delete $arr{id};

    my $description = $arr{description};
    delete $arr{description};

    
    # Check for key/value additions to hashes.
    my $newkey = $arr{newkey};
    my $newvalue = $arr{newvalue};
    delete $arr{newkey};
    delete $arr{newvalue};


    # Get the original record
    my $sth = $dbh->prepare("select * from conf_system where id = ?");
    $sth->execute( $id );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;

    my $type = $ref->{datatype};
    my $dataname = $ref->{dataname};
    my $value_ref = [ ];
    my $name_ref = [ ];


    my $sth = $dbh->prepare("update conf_system set datavalue = ? where id = ?");


    if ( $type eq 'scalar' or $type eq 'scalaronoff' ) {

	foreach my $key ( keys %arr ) { # should only be 1
	    push @$name_ref, $key;
	    push @$value_ref, $arr{$key};
	}
	
	my $d = Data::Dumper->new( $value_ref, $name_ref );
	my $datavalue = $d->Dump;
	$sth->execute( $datavalue, $id );

    } elsif ( $type eq 'array' ) {

	my @newarray = ();
	foreach my $key ( sort { $a <=> $b } keys %arr ) { #
	    if ( defined $arr{$key} ) { # need a value...
		push @newarray, $arr{$key};
	    }
	}
	if ( $newvalue ) {
	    push @newarray, $newvalue;
	}

	$dataname = '*'. $dataname;
	push @$name_ref, $dataname;
	push @$value_ref, \@newarray;

	my $d = Data::Dumper->new( $value_ref, $name_ref );
	my $datavalue = $d->Dump;

	# print qq{Datavalue:$datavalue<br>\n};
	$sth->execute( $datavalue, $id );


    } elsif ( $type eq 'hash' ) {

	my %newhash = ();
	foreach my $key ( keys %arr ) {
	    if ( $arr{$key} ) {
		$newhash{$key} = $arr{$key};
	    }
	}
	if ( $newkey and defined $newvalue ) {
	    $newhash{$newkey} = $newvalue;
	}

	$dataname = '*'. $dataname;
	push @$name_ref, $dataname;
	push @$value_ref, \%newhash;

	my $d = Data::Dumper->new( $value_ref, $name_ref );
	my $datavalue = $d->Dump;

	#print qq{Datavalue:$datavalue<br>\n};
	$sth->execute( $datavalue, $id );


    } elsif ( $type eq 'hashofarray' ) {


	my %newhash = ();

	# Put in an NEW values and remove values from %arr;
	for my $idx ( 0 .. $maxarrayidx ) {
	    my $key = qq{NEW:$idx};
	    if ( $arr{$key} and $newkey ) {
		$newhash{$newkey}[$idx] = $arr{$key};
	    }
	    delete $arr{$key};
	}


	# Loop through all keys and update the hash of arrays
	foreach my $key ( sort keys %arr ) {
	    my ( $hkey, $idx ) = split(/:/, $key);
	    $newhash{$hkey}[$idx] = $arr{$key};
	}

	$dataname = '*'. $dataname;
	push @$name_ref, $dataname;
	push @$value_ref, \%newhash;

	my $d = Data::Dumper->new( $value_ref, $name_ref );
	my $datavalue = $d->Dump;

	$sth->execute( $datavalue, $id );

    } # end of hashofarrays


    # update description
    my $sth = $dbh->prepare("update conf_system set description = ? where id = ?");    
    $sth->execute( $description, $id );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    

    if ( not $DBI::errstr  ) {
	print qq{<h3>$lex{Record} $lex{Updated}</h3>\n};
    } else {
	print qq{<h3>$lex{Error}:$DBI::errstr;</h3>\n};
    }


#    print qq{<p>[ <a href="$eoypage">$lex{Eoy}</a> ]</p>\n};
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="submit" value="$lex{Edit}"></form>\n};

    print qq{</body></html>\n};

    exit;

} # End of writeValues;

