#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $self = 'login.pl';
my $defaultDuration = 90; # minutes 
my $minimumtime = 3; # minutes
my $maximumtime = 120; # minutes

my %lex = ('User Id' => 'User Id',
	   'Password' => 'Password',
	   'Duration' => 'Duration',
	   'Login' => 'Login',
	   'Staff Login' => 'Staff Login',
	   'min' => 'min',
	   'No User Id Found' => 'No User Id Found',
	   'Incorrect Password' => 'Incorrect Password',
	   'Logged In' => 'Logged In',
	   'User' => 'User',
	   'Logout Complete' => 'Logout Complete',
	   'Error' => 'Error',
	   'Continue' => 'Continue',
	   'Close' => 'Close',
	   'Select School' => 'Select School',

	   );


use DBI;
use CGI;
use CGI::Session;

my $q = CGI->new;
my %arr = $q->Vars;

eval require "ppp.conf";
if ( $@ ) {
    print $q->header;
    print $lex{Error}. " global.conf: $@<br>\n";
    die $lex{Error}. " global.conf: $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $q->header; 
    print $lex{Error}. " global.conf: $@<br>\n";
    die $lex{Error}. " global.conf: $@\n";
}

my $dbtype = 'mysql';
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

my $login_result;

if ( $arr{userid} and $arr{password} ) {
    $login_result = doLogin();

} elsif ( $arr{quit} ) {
    doLogout(); 

} else {
    print $q->header;
    printHTMLHeader();
    showStartPage();
}



#----------------
sub showStartPage {
#----------------

    print qq{<center><h3>$lex{'Staff Login'}</h3>\n};

    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="script" value="$arr{script}">\n};

    print qq{<table cellpadding="6" cellspacing="0" border="1">\n};
    print qq{<tr><th class="bra">$lex{'User Id'}</th>\n<th>};
    print qq{<input type="text" name="userid" size="10" value="$arr{user}"></th></tr>\n};

    print qq{<tr><th class="bra">$lex{Password}</th>\n<th>};
    print qq{<input type="password" name="password" size="10"></th></tr>\n};

    print qq{<tr><th class="bra">$lex{Duration}</th><th>};
    print qq{<input type="text" name="duration" size="3" value="$defaultDuration"> };
    print qq{$lex{min}</th></tr>\n};

    print qq{<tr><th class="cn" colspan="2">};
    print qq{<input type="submit" value="$lex{Login}"></th></tr>\n};

    print qq{</table></form></center></body></html>\n};

    exit;

} # end of showStartPage;


#---------------
sub selectSchool {
#---------------

    my @schools = sort keys %dbase; # from global config file.
    # The keys are the databases, the values are the school names.

    print qq{<center><h3>$lex{'Select School'}</h3>\n};

    print qq{<form action="$self" method="post">\n};
    foreach my $key ( keys %arr ) { # pass on rest of values.
	print qq{<input type="hidden" name="$key" value="$arr{$key}">\n};
    }

    print qq{<table cellpadding="6" cellspacing="0" border="1">\n};

    print qq{<tr><td class="bra">$lex{'Select School'}</td>\n<td>};
    print qq{<select name="school"><option></option>\n};
    foreach my $db ( @schools ) {
	print qq{<option value="$db">$dbase{$db}</option>\n};
    }
    print qq{</select></td></tr>\n};


    print qq{<tr><td class="cn" colspan="2">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};

    print qq{</table></form></center></body></html>\n};

    exit;

} # end of selectSchool;




#-----------
sub doLogout {
#-----------

    # $dbtype set in admin.conf / ppp.conf
    my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
				   undef,{Handle => $dbh}) or die CGI::Session->errstr;
    $session->delete;
    $session->flush;
    print $q->header;
    printHTMLHeader();

    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<center><h1>$lex{'Logout Complete'}</h1>\n};
    print qq{</center></body></html>\n};

    exit;

} # end of doLogout




#----------
sub doLogin {
#----------

    #print $q->header;
    #foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
				undef,{Handle => $dbh}) or die CGI::Session->errstr;

    # Check password/userid against database (-1 no user, -2 wrong password;
    my $error = checkPassword( $arr{userid}, $arr{password} );

    if ($error == -1){
	print $q->header; 
	printHTMLHeader();
	print qq{<center><h2>$lex{'No User Id Found'}</h2>\n};
	showStartPage();
    }
    if ($error == -2){ 
	print $q->header;
	printHTMLHeader();
	print qq{<center><h2>$lex{'Incorrect Password'}</h2>\n};
	showStartPage();
	
    }

    $cookietime = checkCookieTime( $arr{duration} );

    # Get School database 
    my $schooldbase;
    if ( not $arr{school} ) { # no school selected by central staff

	# get school dbase value from record.
	my $sth = $dbh->prepare("select schooldbase from educator
         where userid = ?");
	$sth->execute( $arr{userid} );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	$schooldbase = $sth->fetchrow;
    
	# if central office staff, let them select the school (and pass value)
	if ( $schooldbase eq 'per' ) {
	    print $q->header;
	    selectSchool();
	}
    } else { # central office staff have a passed value for the school.
	$schooldbase = $arr{school};
    }



    # Set values for userid, logged_in and duration in session
    $session->param('logged_in','1');
    $session->expire('logged_in',$cookietime);
    $session->param('userid',$arr{userid});
    $session->param('duration', $cookietime );
    # The duration value in the session has cookie markup: +20m format.

    $session->param('schooldbase', $schooldbase );

    # Now print page header...
    print $session->header;
    
    printHTMLHeader();
    print qq{<h3>$lex{'Staff Login'}</h3>\n};

    my $sth = $dbh->prepare("select firstname, lastname from educator 
     where userid = ?");
    $sth->execute( $arr{userid} );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ($firstname, $lastname) = $sth->fetchrow;
    
    print qq{$firstname $lastname<br>$lex{'Logged In'}<br>\n};

    print qq{\n<p><form><input type="hidden" name="none">\n};
    print qq{<input type="button" value="$lex{Close}" };
    print qq{onClick="parent.close()"></form></p>};

    if ( $arr{script} ) {
	print qq{<form action="$arr{script}" method="post">\n};
	print qq{<input type="submit" value="$lex{Continue}">\n};
	print qq{</form>};
    }
    print qq{</body></html>\n};

    exit;

}


#----------------
sub checkPassword {
#----------------

    my ($userid, $password) = @_;
    
    if (not $userid){ return -1;}
    if (not $password){ return -2;}

    # Sanitize 
    unless ( $password =~ m#^([\w\d.-@_+]+)$# ) {
      return -2;
    }
    $password = $1;

    #check for presence of userid
    my $sth = $dbh->prepare("select count(userid) from educator where userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my $count = $sth->fetchrow;
    if ($count < 1){ return -1;} # no userid

    #check for presence of correct password and userid
    my $sth = $dbh->prepare("select count( userid ) from educator
     where userid = ? and password = ?");
    $sth->execute( $userid, $password );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $count = $sth->fetchrow;
    if ( $count < 1 ) { return -2; } # not correct password

    return 0; # if all ok...

}


#------------------
sub checkCookieTime {
#------------------

    # defaults set at top of script
    #$defaultDuration $minimumtime $maximumtime 

    my $cookietime;
    my $duration = shift;
    if ( $duration ) { 
	$cookietime = $duration;
    } else { 
	$cookietime = $defaultDuration;
    }

    $cookietime = $minimumtime if $cookietime < $minimumtime; 
    $cookietime = $maximumtime if $cookietime > $maximumtime; 
    $cookietime = "+".$cookietime."m"; # set format
    
    return $cookietime;

}


#------------------
sub printHTMLHeader {
#------------------

    # Print Page Heading
    print qq{$doctype\n<html><head><title>$lex{'Staff Login'}</title>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};

    if ( $arr{user} ) { # if user, jump to the password field.
	print qq{</head><body onload="document.forms[0].elements[2].focus()">\n};
    } else {
	print qq{</head><body onload="document.forms[0].elements[1].focus()">\n};
    }

}
