#! /usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.


my $self = 'attendcheck.pl';


use DBI;
use CGI;
use Number::Format qw(:all);
use Cwd;


eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


# Absent/Late strings for libattend library.
my %lexi = ('Absent' => 'Absent',
	    'Late' => 'Late'
    );



# load attendance library
eval require "libattend.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currdate = "$year-$month-$day";



my $dbtype = 'mysql';
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my %studs; # studs{dbase}{studnum} = record;
my $sth = $dbh->prepare("select * from special");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my $ref = $sth->fetchrow_hashref ) {
    my %r = %$ref;

    my $studnum = $r{studnum};
    if ( $r{studnum_local} ) {
	$studnum = $r{studnum_local};
    }
    
    $studs{ $r{school} }{$studnum} = $ref;

}


# Loop to check.
=head
foreach my $db ( sort keys %studs ) {
    foreach my $studnum ( keys %{ $studs{$db} } ) {
	my %r = %{ $studs{$db}{$studnum} };
	if ( $r{studnum_local} ) {
	    print "LOCAL:$r{studnum_local} - $r{studnum}\n";
	    sleep 2;
	}
	print qq{DB:$db SN:$studnum REC:$r{lastname} $r{firstname}<br>\n};
    }
}
=cut



my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;



# Page Header
my $title = "IEP Attendance Report";

print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};

print qq{$chartype\n</head><body style="padding:1em;"><a name="top"></a>\n};
print qq{[ <a href="$adminpage">Admin</a> \n};
print qq{]\n};

print qq{<h1>$title</h1>\n};



# Loop through each school in turn and check out attendance for each student
$sth = $dbh->prepare("select distinct school from special order by school");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my $db = $sth->fetchrow ) {
    if ( $db eq 'thunderchild' ) { next; } # skip thunderchild

    my $dsn1 = "DBI:$dbtype:dbname=$db";
    my $dbh1 = DBI->connect($dsn1,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;

    my $first = 1;

    # Load Configuration Variables;
    my $sth = $dbh1->prepare("select id, datavalue from conf_system where filename = 'admin'");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while (	my ($id, $datavalue) = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }


    my $startdate = $schoolstart;
    my $enddate = $currdate;

    my $termstart = $g_MTrackTerm{1}{1}{start};
    # print qq{Term Start: $termstart<br>\n};
    $startdate = $termstart;

    
    my %schooldays = mkSchoolDays($startdate, $enddate, $dbh1);
    # These are the school days open in each month, holidays excluded from the count.
    
    # Loop through students in this school and get their attendance.
    my %sort;
    foreach my $studnum ( keys %{ $studs{$db} } ) {
	my %r = %{ $studs{$db}{$studnum} };
	$sort{"$r{lastname}$r{firstname}$studnum"} = $studnum;
    }

    # Student Loop for this school
    foreach my $key ( sort keys %sort ) {
	my $studnum = $sort{$key};

	my %r = %{ $studs{$db}{$studnum} };

	if ( $r{studnum_local} ) {
	    $studnum = $r{studnum_local};
	}

	my $ppd = $g_ppd{ $r{grade} };
	if ( not $ppd ) { 
	    print qq{<div>Missing Attendance Periods Per Day - SN:$studnum $r{firstname} $r{lastname}};
	    print qq{GR:$r{grade} DB:$db - SETTING PPD to 2<br>\n}; 
	    $ppd = 2;
	}


	if ( $first ) { # print heading.
	    print qq{<h3>$dbase{$db} ($db)</h3>\n};
	    print qq{<table cellpadding="3" border="1" cellspacing="0">\n};
	    print qq{<tr><th>Name</th>};
	    foreach my $yrmo ( sort keys %schooldays ) {
		print qq{<th>$yrmo</th>};
	    }
	    print qq{</tr>\n};
	    $first = 0;
	}

	print qq{<tr><td><b>$r{lastname}</b>, $r{firstname}</td>\n};
	    

	# Calculate the days enrolled for this student
	my $ref = calcMonthlyEnrollment( $studnum, $startdate, $enddate, $dbh1);
	my %enrol = %$ref;  # enrol{year-month} = val
	if ( not $ref ) { #student not enrolled.
	    my $skip = scalar keys %schooldays;
	    print qq{<td colspan="$skip" style="background-color:salmon;font-weight:bold;">};
	    print qq{Enrollment Error: No Enrollment this school year</td></tr>\n};
	    next; 
	} 

	my ($currmonth,$prevmonth);

	# Loop through months
	foreach my $yrmo ( sort keys %schooldays ) {

	    my $abslate = calcMonthlyAttendance( $studnum, $yrmo, $ppd, '', \%lexi, $dbh1);
	    my ($absent,$late ) = split(':', $abslate);

	    
	    my $pctAttendance;
	    if ( $enrol{$yrmo}{days} ) {
		my $present = $enrol{$yrmo}{days} - $absent;
		$pctAttendance = round($present * 100 / $enrol{$yrmo}{days}, 1);
	    } else { # no enrollment....
		print qq{<td>No Enrol</td>};
		next; # month
	    }

	    $prevmonth = $currmonth;
	    $currmonth = $pctAttendance;

	    
	    my $color;
	    if ( $pctAttendance > 70 ) { 
		$color = 'lightgreen';
	    } elsif ( $pctAttendance > 49.9 ) {
		$color = 'yellow';
	    } else {
		$color = 'red';
	    }

	    print qq{<td style="background-color:$color;">$pctAttendance</td>\n};

	} # end of year-month loop
	    
	print qq{</tr>\n};


    } # end of student loop

    if ( $first ) {
	print qq{<h4>No Students Found</h4>\n};
    } else {
	print qq{</table><p></p>\n};
    }

    
} # end of school loop

print qq{</body></html>\n};
exit;




#----------------
sub showStartPage {
#----------------

    # Setup Year-Months.
    my @months;
    my %months;


    my ($sy, $sm, $sd) = split('-', $schoolstart); # schoolstart is global var from config.
    my $yrmo = "$sy-$sm";
    push @months, $yrmo;
    $months{$yrmo} = "$s_month[$sm]-$sy";

    for my $i (1..10) {
	my $mo = $sm + $i;
	my $yr = $sy;
	if ( $mo > 12 ) {
	    $mo = $mo - 12;
	    $yr++;
	}

	if ( $yr == $cyear and $mo == $cmonth ) { # done
	    last;
	}
	
	if ( length $mo == 1 ) { $mo = '0'. $mo; }
	my $yrmo = "$yr-$mo";
	push @months, $yrmo;
	$months{$yrmo} = "$s_month[$mo]-$yr";
    }


    # Get the grades
    my @grades;
    my $sth = $dbh->prepare("select distinct grade from student 
      where grade is not NULL and grade != ''");
    $sth->execute;
    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $gr = $sth->fetchrow ) {
	push @grades, $gr;
    }


    # Get the homerooms.
    my @homerooms;

    my $sth1 = $dbh->prepare("select distinct grade from student where homeroom = ? 
      and grade != '' and grade is not NULL");

    my $sth = $dbh->prepare("select distinct homeroom from student where 
     homeroom is not NULL and homeroom != ''");
    $sth->execute;
    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $hr = $sth->fetchrow ) {

	push @homerooms, $hr; # removed checking below
	# Now check for the grade level in this homeroom
#	$sth1->execute($hr);
#	if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
#	while ( my $gr = $sth1->fetchrow ) {
#	    if ( $g_AttendanceEntryMethod{$gr} eq 'homeroom' ) {
#		push @homerooms, $hr;
#		last;
#	    }
#	}
    }


=head   
    my ($cyear,$cmonth, $cday) = split('-', $currdate);
    # Get Previous month end.
    my $startdayjd = julian_day($cyear, $cmonth, '1');
    my $prevendjd = $startdayjd - 1; # end of previous month.
    my @prev = inverse_julian_day( $prevendjd );
    my $prevdate = join('-', @prev);
=cut


    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    print qq{<table cellpadding="3" border="0" cellspacing="0">\n};

    # Report Type
    print qq{<tr><td class="bra">$lex{'Report Type'}</td><td>};
    print qq{<select name="reporttype"><option>HTML</option><option>PDF</option>\n};
    print qq{</select></td></tr>\n};

    # Month
    print qq{<tr><td class="bra">$lex{Select} $lex{Month}</td>};
    print qq{<td class="la"><select name="month"><option></option>\n}; 

    foreach my $mo ( @months ) {
	print qq{<option value="$mo">$months{$mo}</option>\n};
    }
    print qq{</select></td></tr>\n};

    print qq{<tr><td colspan="2"><hr></td></tr>\n};

    
    # Homerooms
    print qq{<tr><td class="bra">$lex{Homeroom}</td>};
    print qq{<td class="la"><select name="homeroom"><option></option>\n}; 

    foreach my $hr ( sort {$a <=> $b} @homerooms ) {
	print qq{<option>$hr</option>\n};
    }
    print qq{</select> $lex{'Blank=All'}</td></tr>\n};


    # Grades
    print qq{<tr><td class="bra"><span style="font-weight:normal;">$lex{or}</span> };
    print qq{$lex{Grade}</td>};
    print qq{<td class="la"><select name="grade"><option></option>\n}; 

    foreach my $gr ( sort {$a <=> $b} @grades ) {
	print qq{<option>$gr</option>\n};
    }
    print qq{</select> $lex{'Blank=All'}</td></tr>\n};

    
    # Spacer
    print qq{<tr><td colspan="2"><hr></td></tr>\n};

    
    # Group
    print qq{<tr><td class="bra">$lex{'Group by'}</td><td><select name="group">};
    print qq{<option value="grade">$lex{Grade}</option>\n};
    print qq{<option value="homeroom">$lex{Homeroom}</option>};
    # print qq{<option value="band">$lex{Band}</option>}
    print qq{</select></td></tr>\n};


    # Show Withdrawn
#    print qq{<tr><td class="bra">$lex{'Show Withdrawn'}</td>\n};
#    print qq{<td><input type="checkbox" name="showwithdrawn" value="1"></td></tr>\n};

    # Spacer
    print qq{<tr><td colspan="2"><hr></td></tr>\n};        
    print qq{<tr><td colspan="2" class="bcn">PDF Options</td></tr>\n};        
    
    # Paper Size
    $defaultpapersize =~ s/paper//; # strip out word paper so lex works; from admin.conf
    my $defpaper = ucfirst( $defaultpapersize );

    print qq{<tr><td class="bra">$lex{'Paper Size'}</td>\n<td>};
    print qq{<select name="papersize">};
    if ( $lex{$defpaper} ) { print qq{<option>$lex{$defpaper}</option>\n}; }
    my @sizes = qw(Letter A4 Legal);
    foreach my $size ( @sizes ) {
	if ( $size eq $defpaper ) { next; }
	print qq{<option>$lex{$size}</option>};
    }
    print qq{</select></td></tr>\n};

    # Font Size
    print qq{<tr><td class="bra">$lex{'Font Size'}</td>\n<td>};
    print qq{<select name="fontsize"><option>10pt</option><option>11pt</option>\n};
    print qq{<option>12pt</option></select></td></tr>\n};

    # Records per page
    print qq{<tr><td class="bra">$lex{'Records per Page'}</td>\n<td>};
    print qq{<input type="text" name="maxrecords" size="4" value="$defmaxrecords"></td></tr>\n};

    # Spacer
    print qq{<tr><td colspan="2"><hr></td></tr>\n};        

    print qq{<tr><td></td>};
    print qq{<td class="la"><input type="submit" value="$lex{Continue}">\n};
    print qq{</td></tr>\n};

    print qq{</table></form>\n};
    print qq{</body></html>\n};

    exit;
}



# Print Table header at top of each table.
#--------------------
sub print_html_header {
#--------------------

    my $heading = "\L\u$group";
    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    print qq{<tr><th colspan="10" style="font-size:130%;font-weight:bold;">$heading: $curroom</th></tr>};

    print qq{<tr><th>$lex{Name}</th><th>$lex{Grade}<br>$lex{Homeroom}</th><th>$lex{Enrolled}</th>\n};
    print qq{<th>$lex{Attended}<br>($lex{Days})</th>\n};

    print qq{<th>$lex{Absent}<br>$lex{Unexec}<br>($lex{Days})</th>};
    print qq{<th>$lex{Absent}<br>$lex{Excuse}<br>($lex{Days})</th>\n};

    print qq{<th>$lex{Absent}<br>$lex{Other}<br>($lex{Days})</th>};
    print qq{<th>$lex{Absent}<br>$lex{Total}<br>($lex{Days})</th>\n};

    print qq{<th>$lex{Late}<br>$lex{Unexec}<br>($lex{Times})</th>};
    print qq{<th>$lex{Late}<br>$lex{Excuse}<br>($lex{Times})</th></tr>\n};

}


#-------------------
sub print_tex_header {
#-------------------

    my $heading = "\L\u$group";
    print TEX "\\begin{tabular}{|p{5cm}";
    for (1..8){ print TEX "|p{1.4cm}";}
    print TEX "|}\n\\hline\\rowcolor[gray]{0.85}{\\it $heading:} ";
    print TEX "{\\bf\\large $curroom} \n";
    print TEX "& {\\bf ". $lex{Enrolled}. ' ('. $lex{Days}. ")} & {\\bf ";
    print TEX $lex{Attended}. ' ('. $lex{Days}. ")} & ";
    print TEX "{\\bf ". $lex{Absent}. ' '. $lex{Unexec}. ' ('. $lex{Days};
    print TEX ")} &{\\bf ". $lex{Absent}. ' '. $lex{Excuse}. ' ('. $lex{Days}. ")} & ";

    print TEX "{\\bf ". $lex{Absent}. ' '. $lex{Other}. ' ('. $lex{Days}. ")} &{\\bf ";
    print TEX $lex{Absent}. ' '. $lex{Total}. ' ('. $lex{Days}. ")} & \n";
    print TEX "{\\bf ". $lex{Late}. ' '. $lex{Unexec}. ' ('. $lex{Times}. ")} &{\\bf ";
    print TEX $lex{Late}. ' '. $lex{Excuse}. ' ('. $lex{Times}. ")}\\\\ \\hline\n";
}


#--------------------------
sub print_html_class_totals {
#--------------------------

    if ( not $totschooldays ) { 
	print qq{$lex{Error}: $lex{'Total School Days not calculated'}!\n}; 
	return;
    }

    $totpercent = ($totattended/$totschooldays) * 100;

    $totabsentperun = ($totabsentdayun/$totschooldays) * 100;
    $totabsentperex = ($totabsentdayex/$totschooldays) * 100;
    $tototherper = ($totother/$totschooldays) * 100;
    $totabsentpertot = ($totabsenttot/$totschooldays) * 100;

    $totlateperun = ($totlatedayun/$totschooldays) * 100;
    $totlateperex = ($totlatedayex/$totschooldays) * 100;

    # print out stats for this class
    print qq{<tr class="gray"><td class="bra">$lex{Total} - ($lex{'Student-Days'})</td><td></td>\n};

    print "<td>". format_number($totschooldays,2,2). "</td>";
    print "<td>". format_number($totattended,2,2). "<br> ("; 
    print format_number( $totpercent, 2,2 );
    print "%)</td>\n";

    print "<td>". format_number($totabsentdayun,2,2). "<br>("; 
    print format_number( $totabsentperun, 2, 2);
    print "%)</td>\n";

    print "<td>". format_number($totabsentdayex,2,2). "<br>("; 
    print format_number( $totabsentperex, 2,2);
    print "%)</td>\n";

    print "<td>". format_number($totother,2,2). "<br>("; 
    print format_number( $tototherper, 2, 2 );
    print "%)</td>\n";

    print "<td>". format_number($totabsenttot,2,2). "<br>("; 
    print format_number( $totabsentpertot, 2,2);
    print "%)</td>\n";

    print "<td>". format_number($totlatedayun,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $totlateperun, 2,2);
    print "%)</td>\n";

    print "<td>". format_number($totlatedayex,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $totlateperex, 2);
    print "%)</td>\n";

    print "</tr></table>\n";

}


#-------------------------
sub print_tex_class_totals {
#-------------------------
    if (not $totschooldays){
 
	print $lex{Error}. "! ". $lex{'Total School Days not calculated'};
	return;
    }

    $totpercent = ($totattended/$totschooldays) * 100;

    $totabsentperun = ($totabsentdayun/$totschooldays) * 100;
    $totabsentperex = ($totabsentdayex/$totschooldays) * 100;
    $tototherper = ($totother/$totschooldays) * 100;
    $totabsentpertot = ($totabsenttot/$totschooldays) * 100;

    $totlateperun = ($totlatedayun/$totschooldays) * 100;
    $totlateperex = ($totlatedayex/$totschooldays) * 100;


    # print out stats for this class
    print TEX "\\rowcolor[gray]{0.90}";
    print TEX "{\\bf Totals (Student-Days)} & ". format_number($totschooldays,2,2). " &\n";
    print TEX format_number($totattended,2,2). " (", format_number( $totpercent, 2,2),"\\%)& \n";
    print TEX format_number($totabsentdayun,2,2). " (", format_number( $totabsentperun, 2,2),"\\%)&\n";
    print TEX format_number($totabsentdayex,2,2). " (", format_number( $totabsentperex, 2,2),"\\%)&\n";
    print TEX format_number($totother,2,2). " (", format_number( $tototherper, 2,2),"\\%)&\n";
    print TEX format_number($totabsenttot,2,2). " (", format_number( $totabsentpertot, 2,2),"\\%)&\n";

    print TEX format_number($totlatedayun,2,2). ' '. $lex{Days}. " (";
    print TEX format_number( $totlateperun, 2,2),"\\%)&\n";

    print TEX format_number($totlatedayex,2,2). ' '. $lex{Days}. " (";
    print TEX format_number( $totlateperex, 2,2),"\\%)\\\\";
    print TEX "\\hline\\end{tabular}\n\n\\bigskip\n\\bigskip\n";

    print TEX "\\underline{\\hspace{2.5in}} \n\n";
    print TEX "{\\small Teacher}\n";

    print TEX "\\newpage\n\n";

}


#----------------------
sub print_tex_doc_start {
#----------------------

    # Set Paper Size, text width and height
    my ( $papersize, $textwidth, $textheight );

    if ( $arr{papersize} eq $lex{Letter} ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{Legal} ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{A4} ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    }


    print TEX "\\documentclass[$arr{fontsize}, $papersize, oneside]{article}
\\usepackage{array,colortbl,inputenc}
$a_latex_header
%\\renewcommand{\\familydefault}{\\sfdefault}
\\pagestyle{empty}
\\setlength{\\textwidth}{$textwidth}
\\setlength{\\textheight}{$textheight}
\\setlength{\\hoffset}{-38mm}
\\setlength{\\voffset}{-20mm}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0pt}
\\setlength{\\parindent}{0pt}
\\setlength{\\tabcolsep}{5pt}
\\setlength{\\extrarowheight}{3pt}
\\pagestyle{headings}
\\markright{$schoolname \\hfill ". $lex{'Monthly Attendance Report'}.
" -- $month[$month] $year -- ". $lex{Pg}. "  }
\n\n
\\begin{document}\n";
} # End of Print Tex Doc Start


#----------------
sub print_tex_end {
#----------------
    print TEX "\\hline\\end{tabular}\n\n\\end{document}\n";
    close TEX;

    system("$pdflatex $fileName > pdflog$$.txt");
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm $shortname.*");
    
    print qq{<h1><a href="$webdownloaddir/$shortname.pdf">\n};
    print qq{$lex{'View/Download'} $lex{'Monthly Attendance Report'}</a></h1>\n};
    
    print qq{<p>[ \n};
    if ( not $teachermode ) {
	print qq{ <a href="$attpage">$lex{Attendance}</a> | \n};
    }
    
    print qq{ <a href="$webdownloaddir/pdflog$$.txt">};
    print qq{$lex{'View Log File'}</a> ]</p>\n};
    print qq{</body></html>\n};

} # Print Tex End



#--------------------
sub print_html_record {
#--------------------

    my ( $studnum, $schooldays ) = @_;

    print qq{<tr><td>$studentname ($studnum) $withdrawdate</td><td>$gr/$hr</td>\n};
    print qq{<td class="bcn">}. format_number($schooldays, 2,2). qq{</td>\n};

    my $att = format_number($attended, 2,2);
    if ( $attended < 0 ) {
	$att = qq{<span style="color:red;font-size:140%">$att</span>};
    } else {
	$att = "<b>$att</b>";
    }
    print "<td>$att</td>\n";

    
    print "<td><b>". format_number($absentdayun, 2,2). "</b> ("; 
    print format_number( $absentperun,2,2);
    print "%)</td>";

    print "<td><b>". format_number($absentdayex,2,2). "</b> ("; 
    print format_number( $absentperex, 2,2);
    print "%)</td>\n";

    print "<td><b>". format_number($otherday,2,2). "</b> (";
    print format_number( $otherper, 2,2); 
    print "%)</td>";

    print "<td><b>". format_number($absentdaytot,2,2). "</b> ("; 
    print format_number( $absentpertot, 2,2 ); 
    print "%)</td>";

    print "<td><b>". format_number($lateun,2,2). "</b> ("; 
    print format_number( $lateperun, 2,2 );
    print "%)</td>";

    print "<td><b>". format_number($lateex, 2,2). "</b> ("; 
    print format_number( $lateperex, 2,2);
    print "%)</td>";
    print "</tr>\n";


} # End of Print HTML Record


#-------------------
sub print_tex_record {
#-------------------

    my ( $lastname, $firstname, $middlename, $studnum, $schooldays ) = @_;

    print TEX "{\\bf $lastname}, $firstname $middlename ($studnum) $withdrawdate & ";
    print TEX format_number($schooldays,2,2). " & ". format_number($attended,2,2). " & ";
    print TEX format_number($absentdayun,2,2). " (", format_number( $absentperun, 2,2), "\\%)& ";
    print TEX format_number($absentdayex,2,2). " (", format_number( $absentperex, 2,2), "\\%) & ";
    print TEX format_number($otherday,2,2). " (", format_number( $otherper, 2,2), "\\%) & ";
    print TEX format_number($absentdaytot,2,2). " (", format_number( $absentpertot, 2,2),"\\%) & ";
    print TEX format_number($lateun,2,2). " (", format_number( $lateperun, 2,2), "\\%) & ";
    print TEX format_number($lateex,2,2). " (", format_number( $lateperex, 2,2), "\\%) \\\\ \\hline\n\n";

}


#-------------------------
sub print_html_grandtotals {
#-------------------------

    
    # Print Grand Totals
    print qq{<a style="margin:1em;" name="grandtotal"></a>\n};
    
    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    
    print "<tr><th colspan=\"9\" style=\"font-size:130%;font-weight:bold;\">$lex{'Grand Totals'}</th></tr>\n";
    print "<tr><th></th><th>". $lex{Enrolled}. '</th><th>'. $lex{Attended};
    print '<br>('. $lex{Days}. ")</th>\n";

    print "<th>". $lex{Absent}. '<br>'. $lex{Unexec}. '<br>('. $lex{Days}. ")</th><th>\n";
    print $lex{Absent}. '<br>'. $lex{Excuse}. '<br>('. $lex{Days}. ")</th>\n";
    print '<th>'. $lex{Absent}. '<br>'. $lex{Other}. '<br>('. $lex{Days}. ')</th><th>';
    print $lex{Absent}. '<br>'. $lex{Total}. '<br>('. $lex{Days}. ")</th>\n";
    print '<th>'. $lex{Late}. '<br>'. $lex{Unexec}. '<br>('. $lex{Times}. ')</th><th>';
    print $lex{Late}. '<br>'. $lex{Excuse}. '<br>('. $lex{Times}. ")</th></tr>\n";

    $grtotpercent = ($grtotattended/$grtotschooldays) * 100;

    $grtotabsentperun = ($grtotabsentdayun/$grtotschooldays) * 100;
    $grtotabsentperex = ($grtotabsentdayex/$grtotschooldays) * 100;
    $grtototherper = ($grtotother/$grtotschooldays) * 100;
    $grtotabsentpertot = ($grtotabsenttot/$grtotschooldays) * 100;
    
    $grtotlateperun = ($grtotlatedayun/$grtotschooldays) * 100;
    $grtotlateperex = ($grtotlatedayex/$grtotschooldays) * 100;

    # print out grand totals
    print qq{<tr class="gray"><td class="bra">$lex{'Grand Totals'} (Student-Days)</td>\n};

    print "<td>". format_number($grtotschooldays,2,2). "</td>";
    print "<td>". format_number($grtotattended,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $grtotpercent, 2,2 );
    print "%)</td>\n";

    print "<td>". format_number($grtotabsentdayun,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $grtotabsentperun, 2,2 ); 
    print "%)</td>\n";
    
    print "<td>". format_number($grtotabsentdayex,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $grtotabsentperex, 2,2 );
    print "%)</td>\n";

    print "<td>". format_number($grtotother,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $grtototherper, 2,2 ); 
    print "%)</td>\n";
    
    print "<td>". format_number($grtotabsenttot,2,2). ' '. $lex{Days}. "<br>("; 
    print format_number( $grtotabsentpertot, 2,2 ); 
    print "%)</td>\n";
    
    print "<td>". format_number($grtotlatedayun,2,2). "<br>("; 
    print format_number( $grtotlateperun, 2,2 ); 
    print "%)</td>\n";

    print "<td>". format_number($grtotlatedayex,2,2). "<br>("; 
    print format_number( $grtotlateperex, 2,2 ); 
    print "%)</td>\n";

    print "</tr></table>\n";
    
    print "<p>[ <a href=\"$homepage\">$lex{Main}</a> |\n";
    print "<a href=\"$attpage\">$lex{Attendance}</a> |\n";
    print qq{<a href="#top">$lex{Top}</a> ]</p></body></html>\n};

}


#------------------------
sub print_tex_grandtotals {
#------------------------

    print TEX "\\begin{tabular}{|p{4cm}";
    for (1..8){ print TEX "|p{1.5cm}";}
    print TEX "|}\n\\hline\\rowcolor[gray]{0.85}". $lex{'Grand Totals'}. " &";
    print TEX $lex{Enrolled}. " & ". $lex{Attended}. " (". $lex{Days}. ") &";
    print TEX $lex{Absent}. ' '. $lex{Unexec}. " (". $lex{Days}. ") & ". $lex{Absent};
    print TEX ' '. $lex{Excuse}. " (". $lex{Days}. ") &";
    print TEX $lex{Absent}. ' '. $lex{Other}. " (". $lex{Days}. ") & ". $lex{Absent};
    print TEX ' '. $lex{Total}. " (". $lex{Days}. ") &";
    print TEX $lex{Late}. ' '. $lex{Unexec}. " (". $lex{Times}. ") & ". $lex{Late};
    print TEX ' '. $lex{Excuse}. " (". $lex{Times}. ")\\\\ \\hline\n";

    $grtotpercent = ($grtotattended/$grtotschooldays) * 100;

    $grtotabsentperun = ($grtotabsentdayun/$grtotschooldays) * 100;
    $grtotabsentperex = ($grtotabsentdayex/$grtotschooldays) * 100;
    $grtototherper = ($grtotother/$grtotschooldays) * 100;
    $grtotabsentpertot = ($grtotabsenttot/$grtotschooldays) * 100;
    
    $grtotlateperun = ($grtotlatedayun/$grtotschooldays) * 100;
    $grtotlateperex = ($grtotlatedayex/$grtotschooldays) * 100;


    # print out grand totals
    print TEX $lex{'Grand Totals'}. " (". $lex{'Student-Days'}. ") & ";
    print TEX format_number($grtotschooldays,2,2). " &";

    print TEX format_number( $grtotattended,2,2). q{ }. $lex{Days}. " (";
    print TEX format_number( $grtotpercent, 2,2 ). "\\%)&\n";

    print TEX format_number( $grtotabsentdayun,2,2). q{ }. $lex{Days}. " (";
    print TEX format_number( $grtotabsentperun, 2,2 ). "\\%)&\n";

    print TEX format_number($grtotabsentdayex,2,2). q{ }. $lex{Days}. " (";
    print TEX  format_number( $grtotabsentperex, 2,2 ),"\\%)&\n";

    print TEX format_number($grtotother,2,2). q{ }. $lex{Days}. " (";
    print TEX format_number( $grtototherper, 2),"\\%)&\n";

    print TEX format_number($grtotabsenttot,2,2). q{ }. $lex{Days}. " (";
    print TEX format_number( $grtotabsentpertot, 2,2),"\\%)&\n";

    print TEX format_number($grtotlatedayun,2,2). q{ }. " (";
    print TEX format_number( $grtotlateperun, 2,2),"\\%)&\n";

    print TEX format_number($grtotlatedayex,2,2). q{ }. " (";
    print TEX format_number( $grtotlateperex, 2,2),"\\%)\\\\ \n";


} # End of Print_tex_grandtotals

