#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

my %lex = ( 'Main' => 'Main',
	    'Error' => 'Error',

	    );

#  This file is part of Open Admin for Schools.

# A Library of XML Routines for use with Sask Ed XML Schema

#-------------------------------
# Updates:
# Oct 28/04: $nativeschool = 1 in admin.conf is required in order to generate
#  native demographics information. Otherwise not sent.

#-------------------------------

# Assume the use of the following in the calling routines.
#use XML::Writer;
# Read config variables
#require "../../etc/admin.conf" || die "Cannot read admin.conf!";
#$wr = new XML::Writer(OUTPUT => $output, DATA_MODE => '1',DATA_INDENT => '2');

# $schoolnumber is currently global, must now be passed on command line.

# This is removed in all functions, but mkStudentCourseHistory .... also use Data::UUID is removed.
#my $ug = new Data::UUID;
#my $refid = $ug->create_hex();
#$refid =~ s/^0x//;  # strip leading 0x indicating hex encoded string



# ============ Subroutines Start Here.===============

#----------------------------
sub mkStudentSchoolEnrollment {
#----------------------------

    my ( $seiRef, $schoolnumber, $sr_ref ) = @_;

    my %sr = %{ $sr_ref};

    my $refid = $sr{studnum}; # Local Student Number is the refid.

    $wr->startTag( 'StudentSchoolEnrollment', RefId=>$refid );
    mkStudentIdentification( $sr{provnum}, $sr{birthdate} ); # pass provnum and bdate 
    mkStudentInfo( \%sr ); # pass student record hash reference
    $wr->dataElement('SchoolId',$schoolnumber);

    # Needs MembershipType, EntryDate, EntryCode, Grade, PrevProv,PrevCountry,
    #  ExitDate, ExitType,

    mkSchoolEnrollmentInfo( $seiRef );

    my $myprog;
    if ( not $sr{program} ){ # program field empty
	$myprog = $program; # from admin.conf
    } else {
	$myprog = $sr{program};
    }

    $wr->dataElement('DeptAssignedProgramId',$myprog); 
    mkProgramEnrollmentInfo( $currdate );
    $wr->endTag('StudentSchoolEnrollment');
    return $refid;  # return the RefID for other objects to use.

}



#----------------------------- 
sub mkStudentProgramEnrollment {
#-----------------------------

    my ( $studnum, $birthdate, $programid, $startdate, $enddate ) = @_;

    my $refid = $studnum.".1"; 
    # E for enrollment; studentinfo will just use the studnum 

    $wr->startTag('StudentProgramEnrollment', RefId=>$refid);

    mkStudentIdentification($studnum,$birthdate);

    $wr->dataElement('DeptAssignedProgramId',$programid);

    mkProgramEnrollmentInfo($startdate,$enddate);

    $wr->endTag('StudentProgramEnrollment');
    return $refid;
}


#--------------------------
sub mkStudentClassEnrollment {
#--------------------------

    # New - $schoolnumber at the front
    my ($schoolnumber,$refid, $studnum, $birthdate, $classid, $startdate, $enddate, 
	$override, $source, $dropped, $exitdate, $mode) = @_; 

    $wr->startTag('StudentClassEnrollment',RefId=>$refid);

    mkStudentIdentification( $studnum,$birthdate );

    mkClassIdentification( $schoolnumber, $classid, $startdate, $enddate );

    mkClassEnrollmentInfo($override,$source,$dropped,$exitdate,$mode);

#    if ($enddate){
#	mkExamRegistrationInfo($enddate);
#    }

    $wr->endTag('StudentClassEnrollment');
    return $refid;
}


#---------------------
sub mkStudentClassMark {
#---------------------

    # New $schoolnumber added
    my ( $schoolnumber,$refid, $studnum, $birthdate, $classid, $startdate, $enddate, 
	 $mark, $correction) = @_; 

    $wr->startTag('StudentClassMark',RefId=>$refid);

    mkStudentIdentification($studnum,$birthdate);

    mkClassIdentification($schoolnumber, $classid, $startdate, $enddate);

    mkSchoolMarkInfo($mark, $correction);

    $wr->endTag('StudentClassMark');
    return $refid;

}


#----------------
sub mkSchoolClass {
#----------------

    # New $schoolnumber passed in.
    my ( $schoolnumber, $refid, $classid, $startdate, $enddate,
	 $courseid,$teachid,$mode) = @_;

    # Note: $schoolnumber is NOT global now

    $wr->startTag('SchoolClass',RefId=>$refid);

    mkClassIdentification( $schoolnumber, $classid, $startdate, $enddate);

    mkClassInfo( $courseid, $teachid, $mode);

    $wr->endTag('SchoolClass');
    return $refid;
}


#-------------------------
sub mkStudentCourseHistory {
#-------------------------

    my $ug = new Data::UUID;
    my $refid = $ug->create_hex();
    $refid =~ s/^0x//;

    $wr->startTag('StudentCourseHistory',RefId=>$refid);

    mkStudentIdentification($studnum, $birthdate);
    $wr->dataElement('DeptAssignedCourseId',$courseid);

    if ($coursetitle){
	$wr->dataElement('CourseTitle',$coursetitle);
    }

    mkStudentCourseInfo();

    $wr->endTag('StudentCourseHistory');
    return $refid;
}


#====== Lower Level Elements Start Here ==============


#-------------------------
sub mkSchoolEnrollmentInfo {
#-------------------------

    # Needs MembershipType, EntryDate, EntryType, Grade, 
    # PrevProv,PrevCountry, ExitDate, ExitType, ImmersionType,
    # CumFolder, TuitionType, TuitionLength, TuitionCollected, TuitionExchangeProgram

    my ( $sref ) = @_;

    # print "HashRef: $sref \n";
    #($membershipType, $entryDate, $entryType, $grade,$prevProvState,
    # $prevCountry,$exitDate, $exitType) = @$SEIref;
    if ($$sref{MembershipType}){
	$wr->startTag('SchoolEnrollmentInfo',
		      'MembershipType' => $$sref{MembershipType});
    } else {
	$wr->startTag('SchoolEnrollmentInfo');
    }
    
    if ($$sref{EntryDate}){
	$wr->dataElement('EntryDate',$$sref{EntryDate});
    }

    if ($$sref{EntryType}){
	$wr->emptyTag('EntryType','Code' => $$sref{EntryType});
    }

    # Grade: Must be PK,K,01,02, etc.
    if ($$sref{Grade}){
	$grade = uc($$sref{Grade});
	if (length($grade) == 1){ # Add a zero.
	    $grade = '0'.$grade;
	}
	$wr->emptyTag('Grade','Code' => $grade);
    }

    if ($$sref{PrevProvState}){
	$wr->emptyTag('PreviousProvState','Code'=>$$sref{PrevProvState});
    }

    if ($$sref{PrevCountry}){
	$wr->emptyTag('PreviousCountry','Code'=>$$sref{PrevCountry});
    }

    if ($$sref{ExitDate}){
	$wr->dataElement('ExitDate',$$sref{ExitDate});
    }

    if ($$sref{ExitType}){
	$wr->emptyTag('ExitType','Code' => $$sref{ExitType});
    }

    if ( $$sref{ImmersionType} ){
	$wr->emptyTag('ImmersionType','Code' => $$sref{ImmersionType});
    }

    if ( $$sref{CumulativeFolder} ){
	$wr->emptyTag('CumulativeFolder','Code' => $$sref{CumulativeFolder});
    }

    if ( $$sref{TuitionType} ){
	$wr->startTag('TuitionStatus','Code' => $$sref{TuitionType},
		      'Length' => $$sref{TuitionLength},
		      'Collected' => $$sref{TuitionCollected} );

	if ( $$sref{TuitionExchangeProgram} ){
	    $wr->emptyTag('ExchangeProgram','Code' => $$sref{TuitionExchangeProgram});
	}

	$wr->endTag('TuitionStatus');
    }

    $wr->endTag('SchoolEnrollmentInfo');
}


#--------------------
sub mkStudentPersonal {
#--------------------
    # Passed Complete Student Record
    my $sr_ref = shift;
    my %sr = %{ $sr_ref };

    $wr->startTag('StudentPersonal',RefId =>$sr{studnum} );

    mkStudentIdentification( $sr{provnum},$sr{birthdate} ); # pass provnum,bdate 

    mkStudentInfo( \%sr ); # pass entire student record hash ref
    $wr->endTag('StudentPersonal');
}


#--------------------------
sub mkStudentIdentification {
#--------------------------
    # Pass Prov # and Birthdate
    my $deptAssignedPersonId = shift;
    my $birthDate = shift;
    $wr->startTag('StudentIdentification');

    if ($deptAssignedPersonId){
	$wr->dataElement('DeptAssignedPersonId',$deptAssignedPersonId);
    }

    if ( $birthDate and $birthDate ne '0000-00-00' ) {
	$wr->dataElement('BirthDate',$birthDate);
    }

    $wr->endTag('StudentIdentification');
}


#----------------
sub mkStudentInfo { 
#----------------

    # Passed Complete Student Record
    my $sr_ref = shift;
    my %sr = %{ $sr_ref };

    $wr->startTag('StudentInfo');

    # Studnum
    if ( $sr{studnum} ){
	$wr->dataElement('SchoolAssignedPersonId',$sr{studnum} );
    }

    # Health Id (HSN)
    if ( $sr{healthid} ){
	my $healthid = $sr{healthid};
	$healthid =~ s/ //g; # strip spaces, just in case...
	$wr->dataElement('HSN',$healthid );
    }


    # Name
    if ( $sr{lastname} and $sr{firstname} ){ # At least first and lastname
	$wr->startTag('Name',Type => 'Legal'); # Type = Legal Name
	$wr->dataElement('LastName',$sr{lastname} );
	$wr->dataElement('FirstName',$sr{firstname} );
	if ( $sr{initial} ){ # We have an middle name (even  if called 'initial'
	    $wr->dataElement('MiddleName',$sr{initial} );
	}
	$wr->endTag('Name');
    }



    # Start of Demographics - Gender / Treaty
    $wr->startTag('Demographics');

    if ( $sr{sex} ){
	$wr->dataElement('Gender',$sr{sex} );
    }
    
    #~ $nativeschool global used to disable this section.
    # Aboriginal Status
    if ( $sr{ethnic} and $sr{ethnic} ne 'Non Native' and 
	$nativeschool ) { # exists and is native
	my $ethnic;
	if ($sr{ethnic} eq 'Status Native'){
	    $ethnic = 'Treaty';
	} elsif ( $sr{ethnic} eq 'Non-status Native'){
	    $ethnic = 'NS';
	} elsif ( $sr{ethnic} eq 'Metis'){
	    $ethnic = 'Metis';
	} else { $ethnic = ''; }
	$wr->emptyTag('AboriginalStatus','Code' =>$ethnic);
    }

    # Treaty Number (of student); Unique.
    if ( $sr{treaty} and $nativeschool){
	$wr->emptyTag('TreatyStatusNumber','Number'=>$sr{treaty} );
    }

    # Band Affiliation (bandnum)
    if ( $sr{bandnum} and $nativeschool){
	$wr->emptyTag('BandAffiliation','Code' =>$sr{bandnum} );
    }

    # Reserve Residency 
    if ( $sr{reserveres} and $nativeschool){
	my $code = 'On';
	if ( lc( $sr{reservres})  eq 'n' ) { $code = 'Off'; }
	$wr->emptyTag('ReserveResidency','Code' =>$code );
    }

    $wr->endTag('Demographics');


    # Student Address Elements
    if ($sr{address1} or $sr{city1} or $sr{prov1} or $sr{pcode1} ) {
	$wr->startTag('StudentAddress');
	$wr->startTag('Address');

	if ( $sr{address1} ) {
	    $wr->startTag('Street');
	    $wr->dataElement('Line1',$sr{address1});
	    $wr->endTag('Street');
	} elsif ( $sr{address1mailing} ) {
	    $wr->startTag('Street');
	    $wr->dataElement('Line1',$sr{address1mailing});
	    $wr->endTag('Street');
	}

	if ( $sr{city1} ){
	    $wr->dataElement('City', $sr{city1} );
	}

	if ( $sr{prov1} ){
	    $wr->emptyTag('StatePr','Code' =>$sr{prov1} );
	}

	# Write out Country Anyway....
	$wr->emptyTag('Country','Code' => 'CA');

	if ( $sr{pcode1} ){
	    $wr->dataElement('PostalCode',$sr{pcode1} );
	}

	$wr->endTag('Address');
	$wr->endTag('StudentAddress');
    } # End of Address
    
    if ( $sr{hphone1} ){ #home phone exists
	$wr->dataElement('PhoneNumber', $sr{hphone1} ,Format=>'FF',Type => 'Home');
    }
    
    $wr->endTag('StudentInfo');

} # End of mkStudentInfo



#--------------------------
sub mkProgramEnrollmentInfo {
#--------------------------

    my $entryDate =  shift;

    $wr->startTag('ProgramEnrollmentInfo');
    if ($entryDate){
	$wr->dataElement('EntryDate',$entryDate);
    }

    # Dropped from spec
    #if ($exitDate){
    # $wr->dataElement('ExitDate',$exitDate);
    #}

    $wr->endTag('ProgramEnrollmentInfo');
}


#------------------------
sub mkClassIdentification {
#------------------------

    my ($schoolid, $classid, $startdate, $enddate) = @_;

    $wr->startTag('ClassIdentification');

    $wr->dataElement('SchoolId',$schoolid);
    $wr->dataElement('ClassId',$classid);
    $wr->dataElement('StartDate',$startdate);
    $wr->dataElement('EndDate',$enddate);

    $wr->endTag('ClassIdentification');
}


#--------------
sub mkClassInfo {
#--------------

    my ($courseid, $teacherid, $instructionmode) = @_; 

    $wr->startTag('ClassInfo');
    
    $wr->dataElement('DeptAssignedCourseId',$courseid);
    
    $wr->dataElement('EducatorCertificateNumber',$teacherid);
    
    if ($instructionmode){
	$wr->dataElement('ModeOfInstruction',$instructionmode);
    }

    $wr->endTag('ClassInfo');
}

#-------------------------
sub mkExamRegistrationInfo {
#-------------------------

    my ($enddate) = @_;

    # ~~ Comment this out until fixed...
    $wr->startTag('ExamRegistrationInfo');
    # optional and default value in StudentClassEnrollment -,'Type' => 'Class');

    $wr->dataElement('ExamPeriodEndDate',$enddate);

    $wr->endTag('ExamRegistrationInfo');
}

#------------------------
sub mkClassEnrollmentInfo {
#------------------------
    my ($override, $marksource, $dropped, $exitdate, $instructionmode) = @_;

    #  Note: Don't create the object unless something passed.
    #   '$dropped and $exitdate will both be passed as same time.
    if ($override or $marksource or $dropped){
	$wr->startTag('ClassEnrollmentInfo');
    }

    if ($override){
	$wr->startTag('ClassInfoOverride','Override' => $override);
	$wr->emptyTag('ModeOfInstruction','Code' => $instructionmode);
	$wr->endTag('ClassInfoOverride');
    }

    if ($marksource){
	$wr->emptyTag('MarkSource','Code'=>$marksource);
    }

    if ($dropped){
	$wr->emptyTag('Dropped','Code'=>$dropped);
    }

    if ($exitdate){
	$wr->dataElement('ExitDate',$exitdate);
    }

    if ($override or $marksource or $dropped){
	$wr->endTag('ClassEnrollmentInfo');
    }

}

#-------------------
sub mkSchoolMarkInfo {
#-------------------
    my ($mark, $corrcode) = @_;

    if ($corrcode){
	$wr->startTag('SchoolMarkInfo','CorrectionCode' =>$corrcode);
    } else {
	$wr->startTag('SchoolMarkInfo');
    }

    $wr->dataElement('SchoolMark',$mark);

    $wr->endTag('SchoolMarkInfo');
}


#----------------------
sub mkStudentCourseInfo {
#----------------------

    my ( $type, $enddate, $credits, $schoolyear, $dropped, $marks ) = @_;

    $wr->startTag('StudentCourseInfo','Type' =>$type);

    if ($schoolyear){
	$wr->dataElement('SchoolYear',$schoolyear);
    }

    $wr->dataElement('CourseEndDate',$enddate);

    if ($dropped){
	$wr->dataElement('Dropped',$dropped);
    }
    
    if ($mark){
	$wr->dataElement('FinalMark',$mark);
    }

    $wr->dataElement('CreditsEarned',$credits);

    $wr->endTag('StudentCourseInfo');
}


#-------------
sub mkTermInfo {
#-------------
    my ($startdate, $enddate, $description) = @_;
    $wr->startTag('TermInfo');
    
    $wr->dataElement('StartDate',$startdate);
    $wr->dataElement('EndDate',$enddate);
    if ($description){
	$wr->dataElement('Description',$description);
    }

    $wr->endTag('TermInfo');
}


# ========== Message Elements Here ============

#--------------
sub mkSL_Header {
#--------------
    # Global: $schoolnumber
    my ($date, $time, $sourceid, $count) = @_;

    my $timezone = 'UTC-06:00';

    my $shortdate = $date;
    $shortdate =~ s/-//g;
    my $shorttime = $time;
    $shorttime =~ s/://g;
    my $refid = $schoolnumber.".".$shortdate.".".$shorttime.".$$".$count;

    $wr->startTag('SL_Header');

    $wr->dataElement('SL_MsgId',$refid);
    $wr->dataElement('SL_Date',$date);

    $wr->startTag('SL_Time','Zone' => $timezone);
    $wr->characters($time);
    $wr->endTag('SL_Time');

    $wr->dataElement('SL_SourceId',$sourceid);
    $wr->endTag('SL_Header');

    return $refid;
}


# No longer used.
#-----------------------
sub mkSL_UserCredentials {
#-----------------------
    my ($userid, $password) = @_;


    $wr->startTag('SL_UserCredentials');
    
    $wr->dataElement('SL_UserId',$userid);
    $wr->dataElement('SL_Password',$password);

    $wr->endTag('SL_UserCredentials');
}


# Acknowledgement... normally FROM Sask Ed
#-----------
sub mkSL_Ack {
#-----------
    my ($origMsgId, $origSourceId,$status) = @_;

    $wr->startTag('SL_Ack');
    
    $wr->dataElement('SL_OriginalMsgId',$origMsgId);
    $wr->dataElement('SL_OriginalSourceId',$origSourceId);

    $wr->startTag('SL_Status');
    $wr->dataElement('SL_StatusCode',$status);
    $wr->endTag('SL_Status');

    $wr->endTag('SL_Ack');
}


# 1 needed to return true...
1;

