#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson
#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $g_ldapgidnumber = 50; # used to populate the ldap_gidnumber value.


%lex = ( 'Main' => 'Main',
	 'Eoy' => 'Eoy',
	 'Unable to open' => 'Unable to open',
	 'Template' => 'Template',
	 'Staff Member' => 'Staff Member',
	 'Add' => 'Add',
	 'Eoy' => 'Eoy',
	 'Required' => 'Required',
	 'Bold' => 'Bold',
	 'Error' => 'Error',
	 'Preregistration' => 'Preregistration',
	 'Record(s) Stored' => 'Record(s) Stored',
	 'Position' => 'Position',
	 'Grade' => 'Grade',
	 'Homeroom' => 'Homeroom',
	 'Separate with Spaces' => 'Separate with Spaces',
	 'User Id' => 'User Id',
	 'Exists' => 'Exists',
	 'Password' => 'Password',
	 'Contact' => 'Contact',
	 'Access Control(s)' => 'Access Control(s)',
	 'Type' => 'Type',
	 'Percent' => 'Percent',
	 'Teacher' => 'Teacher',
	 'Occupation' => 'Occupation',
	 'Other' => 'Other',
	 'Only' => 'Only',
	 
	 );

use CGI;
use DBI;
use Data::Password qw(:all);  # for password checking...
use Crypt::GeneratePassword qw(:all); # password generation.


# Password config; rest in admin.conf
# For Generation
$g_staffpwd_minfreq = .001;
$g_staffpwd_avgfreq = .001;
$g_staffpwd_lang = 'en'; # only en or de available.


# For Checking
my $g_staffpwd_groups = 0; # turn off character group (uppercase, lowercase, symbols) checking
my $g_staffpwd_following = 0; # turn off following character checking (keyboard, same)
my $maxuidnumber = 10000; # just in case nothing yet in system.

my $maxmulti = 4; # number of positions displayed.

my $self = 'staffadd.pl';


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../../lib/libmeta.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;


# Select table to add to; staff or prereg_staff;
my ($table, $tableview, $multitable );
if ( $arr{tbl} ){ # add to prereg_staff table
    $table = 'prereg_staff';
    $tableview = "<span style=\"color:red;\">". $lex{Preregistration}. 
	'</span>'. q{ }. $lex{'Staff Member'};
    delete $arr{tbl};
    $multitable = 'prereg_staff_multi';
} else {
    $table = 'staff';
    $tableview = $lex{'Staff Member'};
    $multitable = 'staff_multi';
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Print Page Header
my $title = "$lex{Add} $lex{'Staff Member'}"; 
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">$lex{Main}</a> |\n";
print " <a href=\"$eoypage\">$lex{Eoy}</a> ]\n";

print "<h1>$lex{Add} $tableview</h1>\n";

if ( $arr{writeflag} ) {
    delete $arr{writeflag};
    writeRecord();
}

print "<p><b>$lex{Bold}</b> = $lex{Required}</p>\n";


# Read in Template
unless (open (FH,"<../../template/staff_inac.tpl")) {
    print $lex{'Unable to open'}. q{ }. $lex{Template},": $!\n";
    die $lex{'Unable to open'}. q{ }. $lex{Template},": $!\n";
}
my $formtext;
{ local $/; $formtext = <FH>; close FH;}


# Create hash for fieldnames from meta.
my $sth = $dbh->prepare("select fieldid, fieldname, required from meta where tableid = ?");
$sth->execute( 'staff' );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my %fieldnames = ();
while ( my ( $fieldid, $fieldname, $required ) = $sth->fetchrow ) {
    if ( $required ) { # has any value
	$fieldname = "<span style=\"font-weight:bold;\">$fieldname</span>";
    }
    $fieldnames{$fieldid} = $fieldname;
}

# Now put replacement text back in.
$formtext =~ s{\<\*(.*?)\*\>}
  { exists( $fieldnames{$1} ) 
	? $fieldnames{$1} 
	: $1
  }gsex;


# Find all field values, so we only wrap forms around them (typically
# faster than doing all fields in the table )
my @fields = ();
while ( $formtext =~ m/\<\@(.*)\@\>/g){
    push @fields, $1;
}

my %rec = ();

# Get Largest Uidnumber.
foreach my $table ( qw( staff student )) {
    my $sth = $dbh->prepare("select max(ldap_uidnumber) from $table");
    $sth->execute;
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my $uidnumber = $sth->fetchrow;
    if ( $uidnumber > $maxuidnumber ) {
	$maxuidnumber = $uidnumber;
    }
}
$maxuidnumber++; # increment for next one.


# Now set password, other config values at top of script
my $pwd = word( $g_staffpwd_minlen, $g_staffpwd_genlen,
		$g_staffpwd_lang, $g_staffpwd_signs,
		$g_staffpwd_caps, $g_staffpwd_minfreq,
		$g_staffpwd_avgfreq );


$rec{password} = $pwd;
$rec{ldap_gidnumber} = $g_ldapgidnumber; # at top of file...
$rec{ldap_uidnumber} = $maxuidnumber;



# get replacement values for fields
my %values = ();
foreach my $fieldid ( @fields ) {
    $values{$fieldid} = metaInputField(
	'staff',$fieldid, $rec{ $fieldid }, $dbh,'' ); # no passed value, no preheader

}



# now put field values back into $formtext variable...
$formtext =~ s{ \<\@(.*?)\@\> }
  { exists($values{$1}) 
	? $values{$1} 
	: "$values{$1}-$1"
  }gsex;


# print top of form, then $text, then bottom of page.
print "<form action=\"$self\" method=\"post\"> 
<input type=\"hidden\" name=\"writeflag\" value=\"1\">\n";

if ( $table eq 'prereg_staff' ) {
    print "<input type=\"hidden\" name=\"tbl\" value=\"prereg\">\n";
}

print qq{<span style="float:left;background-color:#DDD;padding:0.3em;border:1px solid black;">};
print qq{<input type="submit" value="$lex{Add} $lex{'Staff Member'}">\n};
print qq{</span><br clear="left">\n};


print $formtext,"\n";

# Now do the position, homeroom, grade, and access multivalues.
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" style=\"float:left;margin:0 2em;\">\n";
for my $i (1 .. $maxmulti ) {
    print "<tr><td class=\"ra\">". $lex{Position}. " $i</td><td><select name=\"position$i\"><option></option>\n";
    foreach my $pos ( @g_position ) { print "<option>$pos</option>"; }
    print "\n</select></td></tr>\n";
}

print qq{<tr><td colspan="2" style="background-color:#888;color:white;">};
print qq{$lex{Homeroom} $lex{Teacher} <b>$lex{Only}</b> (Subject Teachers have NO values here)</td></tr>\n};

print "<tr><td class=\"ra\">$lex{Grade}</td><td><input type=\"text\" name=\"grade\" size=\"12\"> ";
print "$lex{'Separate with Spaces'}</td></tr>\n";

print "<tr><td class=\"ra\">$lex{Homeroom}</td><td><input type=\"text\" name=\"homeroom\" size=\"12\"> ";
print "$lex{'Separate with Spaces'}</td></tr>\n";

print qq{<tr><td colspan="2" style="background-color:#888;color:white;">};
print qq{&nbsp;</td></tr>\n};

print "<tr><td class=\"ra\">$lex{'Access Control(s)'}</td><td><input type=\"text\" name=\"access\" size=\"12\"> ";
print "$lex{'Separate with Spaces'}</td></tr>\n";



# First Nations Job Description
# Load FTE
my $sth = $dbh->prepare("select defaultvalue from meta where tableid = ? and fieldid = ?");
$sth->execute('student_inac','fte' );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $default = $sth->fetchrow;
my %fte = split(/\s+/, $default);
foreach my $key ( keys %fte ) { $fte{$key} =~ s/\_/ /g; }

# Load OccupationTypes
$sth = $dbh->prepare("select defaultvalue from meta where tableid = ? and fieldid = ?");
$sth->execute('staff','occupations' );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $default = $sth->fetchrow;
# print "DEF:$default<br>\n";
my %jobtype = split(/\s+/, $default);
foreach my $key ( keys %jobtype ) { $jobtype{$key} =~ s/\_/ /g; }


print "<tr><td colspan=\"2\">";

print "<div><h2>AANDC - Job Description</h2></div>\n";
print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";

print "<tr><th>$lex{Occupation} $lex{Type}</th><th>$lex{Percent}<br>1-100</th></tr>\n";

for my $i (1.. $maxmulti ) {

    # Job Type
    print "<tr><td class=\"la\"><select name=\"$i:type:\"><option></option>\n";
    foreach my $key (sort keys %jobtype ) { print "<option value=\"$key\">$jobtype{$key}</option>"; }
    print "</select></td>\n";

    # Percent
    print "<td class=\"cn\"><input type=\"text\" name=\"$i:percent:\" size=\"4\"></td></tr>\n";

}

print "</table>\n</td></tr><tr><td colspan=\"2\"></td></tr>\n"; 
# end of Job Entry Values



print "<tr><td colspan=\"2\" class=\"cn\" style=\"background-color:#DDD;\"><input type=\"submit\" value=\"";
print "$title\"></td></tr>\n";
print "</table>\n";
print "</form><p>&nbsp;</p></body></html>\n";


#--------------
sub userid2uid { # function to convert a staff userid field value to the ldap_uid value
#--------------

    my $userid = shift;

    return $userid;

}


#--------------
sub writeRecord {
#--------------

    use Data::Dumper;
    $Data::Dumper::Purity = 1;
    $Data::Dumper::Indent = 0;

    # Extract the Occupations Fields, if present
    my %occ = (); # occupation structure
    foreach my $key ( sort keys %arr) {
	my @vals = split(/:/, $key );
	if ( $vals[1] ) { # we have some stuff to take care of.
	    if ( $arr{$key} ) { # if a defined value, add to structure
		if ( $#vals == 2 ) {
		    $occ{"$vals[0]"}{"$vals[1]"}{"$vals[2]"} = $arr{$key};
		} else {
		    $occ{"$vals[0]"}{"$vals[1]"} = $arr{$key};
		}
	    }
	    delete $arr{$key};
	}
     }

    if ( %occ ) {
	my $value_ref = [ ];
	my $name_ref = [ ];

	$dataname = '*occ';
	push @$name_ref, $dataname;
	push @$value_ref, \%occ;

	my $d = Data::Dumper->new( $value_ref, $name_ref );
	my $occupations = $d->Dump;
	$arr{'occupations'} = $occupations;
    }


    # Make sure we have userid, password.
    if ( not $arr{userid} or not $arr{password} ) {
	print "<h1>$lex{Required}: $lex{'User Id'},$lex{Password}</h1>\n";
	print "</body></html>\n";
	exit;
    }

    # Make sure userid doesn't already exist...else warn.
    $sth = $dbh->prepare("select count(*) from $table where userid = ?");
    $sth->execute( $arr{userid} );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    if ( my $count = $sth->fetchrow ) {
	print "<h1>$lex{Error}: $lex{'User Id'} $lex{Exists}</h1>\n";
	print "</body></html>\n";
	exit;
    }

    # Check for password quality using Data::Password module.
    $MINLEN = $g_staffpwd_minlen;  # from admin.conf settings.
    $MAXLEN = $g_staffpwd_maxlen;
    $FOLLOWING = $g_staffpwd_following;
    $GROUPS = $g_staffpwd_groups;
    if ( $g_staffpwd_dictionary ) {
	$DICTIONARY = $g_staffpwd_dictionary;
    } else {
	$DICTIONARY = 0; # turn it off.
    }

    if ( IsBadPassword( $arr{password} )) {
	print "<h1 style=\"color:red;\">$lex{Password} $lex{Error}<br>\n";
	print IsBadPassword( $arr{password} );
	print "</h1></body></html>\n";
	exit;
    }


    # remove multi values from %arr;
    my $grade = $arr{grade};
    delete $arr{grade}; 
    my $homeroom = $arr{homeroom};
    delete $arr{homeroom};
    my $access = $arr{access};
    delete $arr{access};
    for my $i ( 1 .. $maxmulti ) {
	$position{$i} = $arr{"position$i"};
	delete $arr{"position$i"};
    }

#    foreach my $key ( sort keys %position) { print "K:$key V:$position{$key}<br>\n"; }
#    foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }


    my @values = ();
    my @fieldnames = ();

    # push in the passed values (skip blanks since an add process)
    foreach my $key ( sort keys %arr ) { # run through all passed values.
	if ( $arr{$key} ) {    # if we have a value
	    push @values, $arr{$key};
	    push @fieldnames, $key;
	}
    }


    # Set LDAP User Id; Function used in case something tricky wanted, later.
    my $uid = userid2uid( $arr{userid} );
    push @values, $uid;
    push @fieldnames, 'ldap_uid';


    # Join values
    my $fields = join(',', @fieldnames );
    foreach my $val ( @values ) {
	$val = $dbh->quote( $val );
    }
    my $values = join(',', @values );


    # print "Insert Fields: $fields  Values: $values<br>\n";
    $sth = $dbh->prepare("insert into $table ( $fields ) values( $values )");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    # now add the multirecord values.
    $sth = $dbh->prepare("insert into $multitable ( userid, field_name, field_value ) 
      values( ?, ?, ? )");

    my @grades = split / /, $grade;
    foreach my $grade ( @grades ) {
	$sth->execute( $arr{userid}, 'grade', $grade );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    }

    my @hroom = split / /, $homeroom;
    foreach my $hr ( @hroom ) {

	if ( length($hr) > 8 ) { 
	    print "<h1 style=\"color:red;\">$lex{Error}: $lex{Homeroom} $hr $lex{Length} > 8</h1>\n";
	}

	$sth->execute( $arr{userid}, 'homeroom', $hr );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    }

    # Add Access control values.
    my @access = split / /, $access;
    foreach my $ac ( @access ) {
	$sth->execute( $arr{userid}, 'access', $ac );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    }


    for my $i ( 1 .. $maxmulti ) {
	if ( $position{$i} ) { # we have a position value
	    $sth->execute( $arr{userid}, 'position', $position{$i} );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	}
    }

    if (not $DBI::errstr ) {
	print "<h3>";
	print $lex{'Record(s) Stored'};
	print ".</h3></p>";
    } else {
	print "<h3>".$lex{Error}. q{ };
	print $lex{Contact}. " $adminname - ";
	print "<a href=\"mailto:$adminemail\">$adminemail</a>\n";
	print " $DBI::errstr";
	print "</h3>\n";
    }

    print "<p>[ <a href=\"$homepage\">$lex{Main}</a> |\n";
    print "<a href=\"$eoypage\">$lex{Eoy}</a> | \n";

    if ( $table ne 'staff' ) {
	print "<a href=\"$self?tbl=prereg\">";
	print "$lex{Add}<span style=\"color:red;\">$lex{Preregistration}</span> ";
	print "$lex{'Staff Member'}</a> ]\n";

    } else {
	print "<a href=\"$self\">$title</a> ]\n";
    }

    print "</p></center></body></html>\n";

    exit;

}
