#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Edit' => 'Edit',
	   'Name' => 'Name',
	   'Update' => 'Update',
	   'Score' => 'Score',
	   'No User Id' => 'No User Id',
	   'No Password' => 'No Password',
	   'Please Log In' => 'Please Log In',
	   'Record(s) Updated' => 'Record(s) Updated',
	   'Edit not allowed' => 'Edit not allowed',
	   'Scores' => 'Scores',
	   'Date' => 'Date',
	   'Close' => 'Close',
	   'Common Math Assessment' => 'Common Math Assessment',
	   'Outcome' => 'Outcome',
	   'Test' => 'Test',

	   );

use DBI;
use CGI;
use CGI::Session;


my $self = 'cmathEdit.pl';


# NOW in config file math.conf
# my @phases = qw(Emergent Matching Quantifying Partitioning Factoring Operating);
# my @seasons = qw(Sep-2012 Nov-2012 Apr-2013 Sep-2013 Nov-2013 Apr-2014 Sep-2014 Nov-2014);

# Configuration Settings -----------------------
my $allow_author = 1; # allow author to edit
my $allow_cma = 1; # allow user with math access code to edit
#-----------------------------------------------

my $q = new CGI;
my %arr = $q->Vars;

my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Session Setup Here
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

# Get/Set  Session Values (a defined userid means it was passed)
if ( $arr{userid} ){ # we want to login, passed userid/password pair.

    # Check password/userid against database (-1 no user, -2 wrong password);
    my $error = checkPassword($arr{userid}, $arr{password});
    if ($error == -1){ print $q->header( -charset, $charset ); login($lex{'No User Id'}); }
    if ($error == -2){ print $q->header( -charset, $charset ); login($lex{'No Password'}); }

    $cookietime = checkCookieTime( $arr{duration} );

    # Set values for userid and logged_in in session
    $session->param( 'logged_in','1');
    $session->expire( 'logged_in', $cookietime );
    $session->param( 'userid',$arr{userid} );
    $session->param( 'duration',$cookietime );

    $userid = $arr{userid};


} else { # check logged_in value in session

    if ( not $session->param('logged_in') ){
	$userid = $session->param('userid');
        print $q->header( -charset, $charset );
        print $lex{'Please Log In'}. "</body></html>\n"; 
        exit;
    }
    # Ok, we have a login. Values below we have in environment.

    $userid = $session->param('userid');
    $duration = $session->param('duration');

    if ( not ($duration =~ /\+/)) { # if not in +20m format, do so.
	$duration = checkCookieTime( $duration );
    }

    $session->expire('logged_in', $duration );


} # End of check for logged_in value

print $session->header( -charset, $charset );
# Session Setup End


# Check for fsim access code
my $sth = $dbh->prepare("select field_value from staff_multi 
  where field_name = 'access' and userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

my $accesscma;
while ( my $val = $sth->fetchrow ) {
    if ( uc $val eq 'CMA' ) { $accesscma = 1; }
    # print "VAL:$val ACCESS:$accesscma<br>\n";
}



# Page Header
my $title = "$lex{Edit} $lex{'Common Math Assessment'} $lex{Scores}";
print "$doctype\n<html><head><title>$title</title>";
print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";

if ( not $arr{page} ) { # date widget
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}


print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";
print "[ <a href=\"$tchpage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    editTest( $arr{testid} );

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    writeTest();
}


#-------------
sub editTest {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $testid = shift;

    # Get Reading Test fields.
    my $sth = $dbh->prepare("select * from mathca_scores where id = ?");
    $sth->execute( $testid );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;
    my %r = %$ref;

    my $studnum = $ref->{studnum};

    # Access Controls
    my $failflag;
    if ( not $allow_author and not $allow_cma ) { $failflag = 1; } # nobody edits
    elsif ( $allow_author and $ref->{tauthor} ne $userid ) { $failflag = 1; } # not author

    if ( $allow_cma and $accesscma ) { $failflag = 0; } # allow override of author failure for catalyst.

    if ( $failflag ) {
	print "<h3>$lex{'Edit not allowed'}</h3>\n";
	print "</body></html>\n";
	exit;
    }

    # Get Student Name, Grade
    my $sth1 = $dbh->prepare("select lastname, firstname, grade from student where studnum = ?");
    $sth1->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $grade ) = $sth1->fetchrow;
    # print "Name: $lastname $firstname<br>\n";


    # Get Outcomes for this Grade
    my %outcomes;
    my $sth = $dbh->prepare("select oid, odesc from mathca_outcomes where grade = ?");
    $sth->execute( $grade );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while ( my ($oid, $odesc ) = $sth->fetchrow ) {
	$outcomes{$oid} = $odesc;
    }


    # Print Form Heading
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"id\" value=\"$testid\">\n";

    # Print Table Header and Heading values
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    print qq{<tr><td colspan="2" class="la">\n};
    print qq{<input type="submit" value="$lex{Update}"></td></tr>\n};

    # Name
    print qq{<tr><td class="bra">$lex{Name}</td><td class="la"><b>$lastname</b>, $firstname</td></tr>\n};

    # Test Date
    print qq{<tr><td class="bra">$lex{Test} $lex{Date}</td>};
    print qq{<td><input type="text" name="tdate" id="date" size="10" value="$r{tdate}">\n};
    print qq{<button type="reset" id="start_trigger">...</button>\n};
    print qq{</td></tr>\n};

    # Pretest or Posttest
    print qq{<tr><td class="bra">Pretest or Posttest</td><td class="la"><select name="prepost"><option>$r{prepost}</option>\n};
    print qq{<option value="pretest">Pre-Test</option><option value="posttest">Post-Test</option></select></td></tr>\n};

    # Outcome
    print qq{<tr><td class="bra">$lex{Outcome}</td>};
    print qq{<td><select name="outcome"><option value="$r{outcome}">$outcomes{"$r{outcome}"}</option>\n};
    foreach my $oid ( sort keys %outcomes ) {
	print qq{<option value="$oid">$outcomes{$oid}</option>};
    }
    print qq{</td></tr>\n};

    # Score
    print qq{<tr><td class="bra">$lex{Score}</td>};
    print qq{<td class="la">};
    foreach my $score ( 1, 2, 3, 4, 'N/A' ) {
	my $checked;
	if ( $score eq $r{score} ) { $checked = qq{checked="checked"} }
	print qq{<input type="radio" name="score" value="$score" $checked>$score&nbsp;&nbsp;\n};
    }
    print "</td></tr>\n";


    print qq{<tr><td colspan="2" class="la">\n};
    print qq{<input type="submit" value="$lex{Update}"></td></tr>\n};
    print qq{</table></form>\n};

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"date\", // id of the input field
        ifFormat       :    \"%Y-%m-%d\", // format of the input field
        button         :    \"start_trigger\", // trigger for the calendar (button ID)
        singleClick    :    false,        // double-click mode
        step           :    1             // show all years in drop-down boxes 
    });
    </script>\n";

    print qq{</body></html>\n};

    exit;

}


#--------------
sub writeTest {
#--------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
 

    my $sth = $dbh->prepare("update mathca_scores set 
      tdate = ?, outcome = ?, score = ?, prepost = ? where id = ?");  

    $sth->execute( $arr{tdate}, $arr{outcome}, $arr{score}, $arr{prepost}, $arr{id} );

    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    print "<h1>$lex{'Record(s) Updated'}</h1>\n";

    print "<p><form><input type=\"hidden\" name=\"none\">\n";
    print "<input type=\"button\" value=\"$lex{Close}\" ";
    print "onClick=\"parent.close()\"></form></p>";

    print "</body></html>\n";

    exit;

}
