#!/usr/bin/perl
# Configuration Values
my $apacheConfFile = '../install-apache2conf/apache2-divcentral.conf';
my $apacheOutputDir = '/etc/apache2/sites-available';

my $a2ensite = '/usr/sbin/a2ensite';
my $a2enmod = '/usr/sbin/a2enmod';
my $apache2ctl = '/usr/sbin/apache2ctl';
my $htpasswd = '/usr/bin/htpasswd';

my $dbfile = '../sql/div_central_site/central_reading.sql';


print "\nEnter Central Division Site  Name/Directory (single word only): ";
$centraldir = <STDIN>;
chomp $centraldir;

# strip any spaces
$centraldir =~ s/\s//g;
print "Central Directory: $centraldir\n\n";


# Check for existing /opt/openadmin
if ( not -e '/opt/openadmin' ) {
    print "Creating '/opt/openadmin'\n\n";
    mkdir '/opt/openadmin';
}

# Copy central application files into place
my $error = system("cp -r ../../central  /opt/openadmin/$centraldir");
if ( not $error ) {
    print "OA Division Central Site copied to /opt/openadmin/$centraldir\n\n";
}

# Change Ownership
my $error = system("chown -R www-data:www-data /opt/openadmin/$centraldir");
if ( not $error ) {
    print "Ownership for OA Central files changed to www-data:www-data\n\n";
}


# Open Apache Configuration File
unless ( open(FH, "<$apacheConfFile") ) {
	print "Cannot open $apacheConfFile: $!\n";
	die "Cannot open $apacheConfFile: $!\n";
}
# slurp in file
my $apachefile;
{ local $/; $apachefile = <FH>; close FH;}
close FH;

# Get Domain Name
print "\nEnter Domain Name for Division Central site (no http://): ";
$centralsite = <STDIN>;
chomp $centralsite;
print "Central Site: $centralsite\n\n";


# Update Apache File.
$apachefile =~ s/DOMAINNAME/$centralsite/;
$apachefile =~ s/CPASSWORD/$centraldir/g;
$apachefile =~ s/CENTRAL/$centraldir/g;

# Write Apache File into location.
unless ( open(FH, ">$apacheOutputDir/$centraldir.conf") ) {
	print "Cannot open $apacheOutputDir/$centraldir.conf: $!\n";
	die "Cannot open $apacheOutputDir/$centraldir.conf: $!\n";
}

print FH $apachefile;
close FH;


# Turn on Config, Turn on Modules, Restart Apache.
system("$a2ensite $centraldir.conf ");
system("$apache2ctl configtest"); # test config
system("$apache2ctl graceful"); # gracefully restart


# Create the authorization file.
# Check for existing /auth_users
if ( not -e '/etc/apache2/auth_users' ) {
    print "Creating '/etc/apache2/auth_users'\n\n";
    mkdir '/etc/apache2/auth_users';
}

print "\nEnter Central Website User Id: ";
$adminuser = <STDIN>;
chomp $adminuser;
print "\nEnter Central Website User Password: ";
$adminpassword = <STDIN>;
chomp $adminpassword;
print "\nUser:$adminuser  Password:$adminpassword\n";

my $authfile = $centraldir;
$error = system("$htpasswd -cb $authfile $adminuser $adminpassword");
if ( not $error ) {
    print "Apache Authorization File Created\n";
}
$error = system("mv $authfile /etc/apache2/auth_users/");
if ( not $error ) {
    print "Authorization File Moved into place.\n";
}


# Now create the central database and populate it.
# Get MySql Root password
print "\nEnter MySQL Root Password: ";
$mysqlroot = <STDIN>;
chomp $mysqlroot;
print "MySQL Root Password: $mysqlroot\n";


$error = system("mysqladmin -p$mysqlroot -u root create $centraldir");
if ( not $error ) {
    print "MySQL Database Created\n";
}

$error =  system("mysql -p$mysqlroot -u root $centraldir < $dbfile");
if ( not $error ) {
    print "MySQL Database Tables/Data Added\n\n";
}





# Now rewrite the admin.conf, after getting root password, and password for the database.
print "\nEnter School MySQL Password: ";
$schoolmysqlpwd = <STDIN>;
chomp $schoolmysqlpwd;
print "School MySQL Password: $schoolmysqlpwd\n\n";


$error =  system("mysql -p$mysqlroot -u root -e \"grant all on $centraldir.* to $centraldir\@localhost identified by '$schoolmysqlpwd'\"");
if ( not $error ) {
    print "Database user and password added\n";
}


# Load main OA Configuration File, and replace values.
unless ( open(FH, "</opt/openadmin/$centraldir/etc/admin.conf") ) {
	print "Cannot open admin.conf (read): $!\n";
	die "Cannot open admin.conf (read): $!\n";
}
# slurp in file
my $adminfile;
{ local $/; $adminfile = <FH>; close FH;}
close FH;

$adminfile =~ s/USER/$centraldir/;
$adminfile =~ s/DATABASE/$centraldir/;
$adminfile =~ s/PASSWORD/$schoolmysqlpwd/;

$adminfile =~ s/YOURSCHOOL/$centraldir/g;


unless ( open(FH, ">/opt/openadmin/$centraldir/etc/admin.conf") ) {
	print "Cannot open admin.conf (write): $!\n";
	die "Cannot open admin.conf (write): $!\n";
}

print FH $adminfile;
close FH;
print "Done!\n";
