#!/usr/bin/perl
# Configuration Values
my $apacheConfFile = '../install-apache2conf/apache2-iep.conf';
my $apacheOutputDir = '/etc/apache2/sites-available';

my $ieppwd = 'ieppwd'; # teacher password file
my $ieppwdadmin = 'iepadminpwd'; # admin password file.


my $a2ensite = '/usr/sbin/a2ensite';
my $a2enmod = '/usr/sbin/a2enmod';
my $apache2ctl = '/usr/sbin/apache2ctl';
my $htpasswd = '/usr/bin/htpasswd';

my $dbfile = 'iep_5.50.sql';


print "\nEnter IEP Identifier for directory (single word only): ";
$iepdir = <STDIN>;
chomp $iepdir;

# strip any spaces
$iepdir =~ s/\s//g;
print "IEP Directory: $iepdir\n\n";


# Check for existing /opt/openadmin
if ( not -e '/opt/openadmin' ) {
    print "Creating '/opt/openadmin'\n\n";
    mkdir '/opt/openadmin';
}

my $error = system("cp -r ../../iep  /opt/openadmin/$iepdir");
if ( not $error ) {
    print "IEP System copied to /opt/openadmin/$iepdir\n\n";
}

my $error = system("chown -R www-data:www-data /opt/openadmin/$iepdir");
if ( not $error ) {
    print "Ownership for IEP files changed to www-data:www-data\n\n";
}



# Now write Apache 2 configuration
# Load Apache Configuration File
unless ( open(FH, "<$apacheConfFile") ) {
	print "Cannot open $apacheConfFile: $!\n";
	die "Cannot open $apacheConfFile: $!\n";
}
# slurp in file
my $apachefile;
{ local $/; $apachefile = <FH>; close FH;}
close FH;

# Get Domain Name
print "\nEnter Domain Name for IEP site (no http://): ";
$iepdomain = <STDIN>;
chomp $iepdomain;
print "IEP Domain: $iepdomain\n\n";


# Update Apache File.
$apachefile =~ s/IEPDOMAIN/$iepdomain/;
$apachefile =~ s/IEPDIR/$iepdir/g;

$apachefile =~ s/IEPPWDADMIN/$ieppwdadmin/g; # set at top of file
$apachefile =~ s/IEPPWD/$ieppwd/g; # set at top of file


# Write Apache File into location.
unless ( open(FH, ">$apacheOutputDir/$iepdir.conf") ) {
	print "Cannot open $apacheOutputDir/$iepdir.conf: $!\n";
	die "Cannot open $apacheOutputDir/$iepdir.conf: $!\n";
}

print FH $apachefile;
close FH;



# Turn on Config, Restart Apache.
system("$a2ensite $iepdir.conf ");
system("$apache2ctl configtest"); # test config
system("$apache2ctl graceful"); # gracefully restart



# Create the authorization file.
# Check for existing /opt/openadmin
if ( not -e '/etc/apache2/auth_users' ) {
    print "Creating '/etc/apache2/auth_users'\n\n";
    mkdir '/etc/apache2/auth_users';
}


print "\nEnter Shared Teacher User Id: ";
$tchuser = <STDIN>;
chomp $tchuser;
print "\nEnter Admin Website User Password: ";
$tchpassword = <STDIN>;
chomp $tchpassword;
print "\nUser:$tchuser  Password:$tchpassword\n";



print "\nEnter Admin Website User Id: ";
$adminuser = <STDIN>;
chomp $adminuser;
print "\nEnter Admin Website User Password: ";
$adminpassword = <STDIN>;
chomp $adminpassword;
print "\nUser:$adminuser  Password:$adminpassword\n";


# Create admin password files
$error = system("$htpasswd -cb $ieppwdadmin $adminuser $adminpassword");
if ( not $error ) {
    print "Apache Admin Authorization File Created\n";
}
$error = system("mv $ieppwdadmin /etc/apache2/auth_users/");
if ( not $error ) {
    print "Admin Authorization File Moved into place.\n\n";
}

# Create teacher password files
$error = system("$htpasswd -cb $ieppwd $tchuser $tchpassword");
if ( not $error ) {
    print "Apache Teacher Authorization File Created\n";
}
$error = system("mv $ieppwd /etc/apache2/auth_users/");
if ( not $error ) {
    print "Teacher Authorization File Moved into place.\n\n";
}



# Now create the database and populate it.
# Get MySql Root password
print "\nEnter MySQL Root Password: ";
$mysqlroot = <STDIN>;
chomp $mysqlroot;
print "MySQL Root Password: $mysqlroot\n";


$error = system("mysqladmin -p$mysqlroot -u root create $iepdir");
if ( not $error ) {
    print "MySQL Database Created\n";
}

$error =  system("mysql -p$mysqlroot -u root $iepdir < ../sql_iep/$dbfile");
if ( not $error ) {
    print "MySQL Database Tables/Data Added\n\n";
}



# Now rewrite the iep.conf, get password for the database.
print "\nEnter IEP MySQL Password: ";
$iepmysqlpwd = <STDIN>;
chomp $iepmysqlpwd;
print "IEP MySQL Password: $iepmysqlpwd\n";


print "\nEnter Global Directory (database locations) : ";
$globaldir = <STDIN>;
chomp $globaldir;
print "Global Directory: $globaldir\n";


$error =  system("mysql -p$mysqlroot -u root -e \"grant all on $iepdir.* to $iepdir\@localhost identified by '$iepmysqlpwd'\"");
if ( not $error ) {
    print "Database user and password added\n";
}


# Load IEP Configuration File, and replace values.
unless ( open(FH, "</opt/openadmin/$iepdir/cgi/ppp.conf.orig") ) {
	print "Cannot open ppp.conf (read): $!\n";
	die "Cannot open ppp.conf (read): $!\n";
}
# slurp in file
my $adminfile;
{ local $/; $adminfile = <FH>; close FH;}
close FH;

$adminfile =~ s/USER/$iepdir/;
$adminfile =~ s/DATABASE/$iepdir/;
$adminfile =~ s/PASSWORD/$iepmysqlpwd/;
$adminfile =~ s/GLOBALDIR/$globaldir/;

$adminfile =~ s/IEPDIR/$iepdir/g;

# print "ppp.conf: $adminfile\n\n";

unless ( open(FH, ">/opt/openadmin/$iepdir/cgi/ppp.conf") ) {
    print "Cannot open ppp.conf (write): $!\n";
    die "Cannot open ppp.conf (write): $!\n";
}

print FH $adminfile;
close FH;

print "Done!\n";
